#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2013 - 2015 CERN
# Author: Pawel Szostek (pawel.szostek@cern.ch)
# Multi-tool support by Javier D. Garcia-Lasheras (javier@garcialasheras.com)
# Riviera tool added by Josh Smith (joshrsmith@gmail.com)
#
# This file is part of Hdlmake.
#
# Hdlmake is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Hdlmake is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Hdlmake.  If not, see <http://www.gnu.org/licenses/>.
#

"""Module providing support for Aldec Riviera-PRO simulation"""

from __future__ import print_function
from .sim_makefile_support import VsimMakefileWriter

# as of 2014.06, these are the standard libraries
# included in an installation
RIVIERA_STANDARD_LIBS = [
    'ieee', 'std', 'cpld',
    'vl', 'vital95', 'vital2000',
    'synopsys', 'aldec', 'vtl',
    'vtl_dbg', 'assertions', 'ieee_proposed',
    'ovm_2_0_3', 'ovm_2_1_2', 'uvm_1_0p1',
    'uvm_1_1d', 'uvm', 'osvvm',
]

# there are many vendor specific libraries available
# a few of them are listed here
RIVIERA_XILINX_VHDL_LIBRARIES = [
    'cpld',
    'secureip',
    'simprim',
    'unimacro',
    'unisim',
    'xilinxcorelib'
]
RIVIERA_XILINX_VLOG_LIBRARIES = [
    'cpld_ver',
    'secureip',
    'simprims_ver',
    'uni9000_ver',
    'unimacro_ver',
    'unisims_ver',
    'xilinxcorelib_ver'
]

RIVIERA_STANDARD_LIBS.extend(RIVIERA_XILINX_VHDL_LIBRARIES)
RIVIERA_STANDARD_LIBS.extend(RIVIERA_XILINX_VLOG_LIBRARIES)


class ToolRiviera(VsimMakefileWriter):

    """Class providing the interface for Aldec Riviera-PRO simulator"""

    TOOL_INFO = {
        'name': 'Riviera',
        'id': 'riviera',
        'windows_bin': 'vsim.exe',
        'linux_bin': 'vsim'}

    STANDARD_LIBS = RIVIERA_STANDARD_LIBS

    CLEAN_TARGETS = {'clean': ["*.asdb"],
                     'mrproper': ["*.vcd"]}

    def __init__(self):
        super(ToolRiviera, self).__init__()
        self._tool_info.update(ToolRiviera.TOOL_INFO)
        self._standard_libs.extend(ToolRiviera.STANDARD_LIBS)
        self._clean_targets.update(ToolRiviera.CLEAN_TARGETS)

    def _makefile_sim_options(self):
        """Print the Riviera options to the Makefile"""
        vcom_opt = self.manifest_dict.get("vcom_opt", '')
        self.manifest_dict["vcom_opt"] = "-2008 " + vcom_opt
        super(ToolRiviera, self)._makefile_sim_options()



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2013 - 2016 CERN
# Author: Pawel Szostek (pawel.szostek@cern.ch)
# Multi-tool support by Javier D. Garcia-Lasheras (javier@garcialasheras.com)
#
# This file is part of Hdlmake.
#
# Hdlmake is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Hdlmake is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Hdlmake.  If not, see <http://www.gnu.org/licenses/>.
#

"""Module providing support for Altera Quartus synthesis"""

from __future__ import absolute_import
import os
import sys
import logging

from .make_syn import ToolSyn
from hdlmake.util import path as path_mod
from hdlmake.util import shell
from hdlmake.srcfile import (VHDLFile, VerilogFile, SVFile, DPFFile,
                             SignalTapFile, SDCFile, QIPFile, QSYSFile,
                             QSFFile, BSFFile, BDFFile, TDFFile, GDFFile)


class ToolQuartus(ToolSyn):

    """Class providing the interface for Altera Quartus synthesis"""

    TOOL_INFO = {
        'name': 'Quartus',
        'id': 'quartus',
        'windows_bin': 'quartus_sh.exe -t',
        'linux_bin': 'quartus_sh -t',
        'project_ext': 'qpf'}

    STANDARD_LIBS = ['altera', 'altera_mf', 'lpm', 'ieee', 'std']

    _QUARTUS_SOURCE = 'set_global_assignment -name {0} $(sourcefile)'

    SUPPORTED_FILES = {
        SignalTapFile: _QUARTUS_SOURCE.format('SIGNALTAP_FILE'),
        SDCFile: _QUARTUS_SOURCE.format('SDC_FILE'),
        QIPFile: _QUARTUS_SOURCE.format('QIP_FILE'),
        QSYSFile: _QUARTUS_SOURCE.format('QSYS_FILE'),
        DPFFile: _QUARTUS_SOURCE.format('MISC_FILE'),
        QSFFile: _QUARTUS_SOURCE.format('SOURCE_TCL_SCRIPT_FILE'),
        BSFFile: _QUARTUS_SOURCE.format('BSF_FILE'),
        BDFFile: _QUARTUS_SOURCE.format('BDF_FILE'),
        TDFFile: _QUARTUS_SOURCE.format('AHDL_FILE'),
        GDFFile: _QUARTUS_SOURCE.format('GDF_FILE')}

    _QUARTUS_LIBRARY = " -library {0}".format('work')

    HDL_FILES = {
        VHDLFile: _QUARTUS_SOURCE.format('VHDL_FILE') +
                  _QUARTUS_LIBRARY,
        VerilogFile: _QUARTUS_SOURCE.format('VERILOG_FILE') +
                     _QUARTUS_LIBRARY,
        SVFile: _QUARTUS_SOURCE.format('SYSTEMVERILOG_FILE') +
                 _QUARTUS_LIBRARY}

    CLEAN_TARGETS = {'clean': ["*.rpt", "*.smsg", "*.summary",
                               "*.done", "*.jdi", "*.pin", "*.qws",
                               "db", "incremental_db", "$(PROJECT).qsf",
                               "*.qpf"],
                     'mrproper': ["*.sof", "*.pof", "*.jam", "*.jbc",
                                  "*.ekp", "*.jic"]}

    TCL_CONTROLS = {'create': 'project_new $(PROJECT)',
                    'open': 'project_open $(PROJECT)',
                    'project': 'load_package flow\n'
                               '$(TCL_CREATE)\n'
                               'remove_all_global_assignments -name *_FILE\n'
                               'source files.tcl',
                    'bitstream': 'load_package flow\n'
                                 '$(TCL_OPEN)\n'
                                 'execute_flow -compile',
                    'install_source': ''}

    SET_GLOBAL_INSTANCE = 0
    SET_INSTANCE_ASSIGNMENT = 1
    SET_LOCATION_ASSIGNMENT = 2
    SET_GLOBAL_ASSIGNMENT = 3

    PROP_TYPE = {"set_global_instance": SET_GLOBAL_INSTANCE,
                 "set_instance_assignment": SET_INSTANCE_ASSIGNMENT,
                 "set_location_assignment": SET_LOCATION_ASSIGNMENT,
                 "set_global_assignment": SET_GLOBAL_ASSIGNMENT}

    def __init__(self):
        super(ToolQuartus, self).__init__()
        self._tool_info.update(ToolQuartus.TOOL_INFO)
        self._hdl_files.update(ToolQuartus.HDL_FILES)
        self._supported_files.update(ToolQuartus.SUPPORTED_FILES)
        self._standard_libs.extend(ToolQuartus.STANDARD_LIBS)
        self._clean_targets.update(ToolQuartus.CLEAN_TARGETS)
        self._tcl_controls.update(ToolQuartus.TCL_CONTROLS)

    def _makefile_syn_top(self):
        """Update project synthesis variables for Quartus"""
        import re

        def __get_family_string(family=None, device=None):
            """Function that looks for a existing device family name and
            try to guess the value from the device string if not defined"""
            family_names = {
                "^EP2AGX.*$": "Arria II GX",
                "^EP1C.*$": "Cyclone",
                "^EP2C.*$": "Cyclone II",
                "^EP3C.*$": "Cyclone III",
                "^EP4C.*$": "Cyclone IV",
                "^EP4CE.*$": "Cyclone IV E",
                "^EP4CGX.*$": "Cyclone IV GX",
                "^5A.*$": "Arria V",
                "^5S.*$": "Stratix V"}
            if family is None:
                for key in family_names:
                    if re.match(key, device.upper()):
                        family = family_names[key]
                        logging.debug(
                            "Auto-guessed syn_family to be %s (%s => %s)",
                            family, device, key)
            if family is None:
                logging.error("Could not auto-guess device family, please "
                              "specify in Manifest.py using syn_family!")
                sys.exit("\nExiting")
            return family

        family_string = __get_family_string(
            family=self.manifest_dict.get("syn_family", None),
            device=self.manifest_dict.get("syn_device", ''))
        device_string = (self.manifest_dict["syn_device"] +
                         self.manifest_dict["syn_package"] +
                         self.manifest_dict["syn_grade"])
        self.manifest_dict["syn_family"] = family_string
        self.manifest_dict["syn_device"] = device_string
        super(ToolQuartus, self)._makefile_syn_top()

    def _emit_property(self, command, new_property):
        """Emit a formated property for Altera Quartus TCL"""
        property_dict = {
            'what': None,
            'name': None,
            'name_type': None,
            'from_': None,
            'to_': None,
            'section_id': None,
            'tag_': None}
        property_dict.update(new_property)
        words = []
        words.append(dict([(b, a) for a, b in
                     self.PROP_TYPE.items()])[command])
        if property_dict['what'] is not None:
            words.append(property_dict['what'])
        if property_dict['name'] is not None:
            words.append("-name")
            words.append(property_dict['name_type'])
            words.append(property_dict['name'])
        if property_dict['from_'] is not None:
            words.append("-from")
            words.append(property_dict['from_'])
        if property_dict['tag_'] is not None:
            words.append("-tag")
            words.append(property_dict['to_'])
        if property_dict['to_'] is not None:
            words.append("-to")
            words.append(property_dict['to_'])
        if property_dict['section_id'] is not None:
            words.append("-section_id")
            words.append(property_dict['section_id'])
        return ' '.join(words)

    def _makefile_syn_tcl(self):
        """Add initial properties to the Altera Quartus project"""
        command_list = []
        command_list.append(self._tcl_controls["project"])
        command_list.append(self._emit_property(
            self.SET_GLOBAL_ASSIGNMENT,
            {'name_type': 'FAMILY',
            'name': '\\"$(SYN_FAMILY)\\"'}))
        command_list.append(self._emit_property(
            self.SET_GLOBAL_ASSIGNMENT,
            {'name_type': 'DEVICE',
            'name':'\\"$(SYN_DEVICE)\\"'}))
        command_list.append(self._emit_property(
            self.SET_GLOBAL_ASSIGNMENT,
            {'name_type': 'TOP_LEVEL_ENTITY',
            'name': '\\"$(TOP_MODULE)\\"'}))
        self._tcl_controls["project"] = '\n'.join(command_list)
        super(ToolQuartus, self)._makefile_syn_tcl()

    def _makefile_syn_files(self):
        # Insert the Quartus standard control TCL files
        command_list = []
        if "quartus_preflow" in self.manifest_dict:
            path = shell.tclpath(path_mod.compose(
                self.manifest_dict["quartus_preflow"], os.getcwd()))
            if not os.path.exists(path):
                logging.error("quartus_preflow file listed in "
                              + os.getcwd() + " doesn't exist: "
                              + path + ".\nExiting.")
                quit(1)
            preflow = '"' + 'quartus_sh:' + path + '"'
            command_list.append(self._emit_property(self.SET_GLOBAL_ASSIGNMENT,
                                {'name_type': 'PRE_FLOW_SCRIPT_FILE',
                                'name': preflow}))
        if "quartus_postmodule" in self.manifest_dict:
            path = shell.tclpath(path_mod.compose(
                self.manifest_dict["quartus_postmodule"],
                os.getcwd()))
            if not os.path.exists(path):
                logging.error("quartus_postmodule file listed in "
                              + os.getcwd() + " doesn't exist: "
                              + path + ".\nExiting.")
                quit(1)
            postmodule = '"' + 'quartus_sh:' + path + '"'
            command_list.append(self._emit_property(self.SET_GLOBAL_ASSIGNMENT,
                                {'name_type': 'POST_MODULE_SCRIPT_FILE',
                                'name': postmodule}))
        if "quartus_postflow" in self.manifest_dict:
            path = shell.tclpath(path_mod.compose(
                self.manifest_dict["quartus_postflow"], os.getcwd()))
            if not os.path.exists(path):
                logging.error("quartus_postflow file listed in "
                              + os.getcwd() + " doesn't exist: "
                              + path + ".\nExiting.")
                quit(1)
            postflow = '"' + 'quartus_sh:' + path + '"'
            command_list.append(self._emit_property(self.SET_GLOBAL_ASSIGNMENT,
                                {'name_type': 'POST_FLOW_SCRIPT_FILE',
                                'name': postflow}))
        self._tcl_controls["files"] = '\n'.join(command_list)
        super(ToolQuartus, self)._makefile_syn_files()



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2013 CERN
# Author: Pawel Szostek (pawel.szostek@cern.ch)
#
# This file is part of Hdlmake.
#
# Hdlmake is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Hdlmake is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Hdlmake.  If not, see <http://www.gnu.org/licenses/>.

"""Module providing the fetcher for local HDLMake modules"""

from .fetcher import Fetcher


class Local(Fetcher):

    """This class provides the Local fetcher instances"""

    def __init__(self):
        pass

    def fetch(self, module):
        pass

    @staticmethod
    def check_md5sum(path):
        """Get the ID for Local sources... maybe sha256 or md5sum?"""
        pass



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2013 CERN
# Author: Pawel Szostek (pawel.szostek@cern.ch)
#
# This file is part of Hdlmake.
#
# Hdlmake is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Hdlmake is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Hdlmake.  If not, see <http://www.gnu.org/licenses/>.

"""Module providing the stuff for handling SVN repositories"""

from __future__ import absolute_import
import os
import logging
from hdlmake.util import path as path_utils
from .fetcher import Fetcher


class Svn(Fetcher):

    """This class provides the SVN fetcher instances, that are
    used to fetch and handle SVN repositories"""

    def __init__(self):
        pass

    def fetch(self, module):
        """Get the code from the remote SVN repository"""
        fetchto = module.fetchto()
        if not os.path.exists(fetchto):
            os.mkdir(fetchto)
        basename = path_utils.svn_basename(module.url)
        mod_path = os.path.join(fetchto, basename)
        cmd = "cd {0} && svn checkout {1} " + basename
        if module.revision:
            cmd = cmd.format(fetchto, module.url + '@' + module.revision)
        else:
            cmd = cmd.format(fetchto, module.url)
        success = True
        logging.info("Checking out module %s", mod_path)
        logging.debug(cmd)
        if os.system(cmd) != 0:
            success = False
        module.isfetched = True
        module.path = mod_path
        return success

    @staticmethod
    def check_svn_revision(path):
        """Get the revision number for the SVN repository at path"""
        svn_cmd = "svn info 2>/dev/null | awk '{if(NR == 5) {print $2}}'"
        return Fetcher.check_id(path, svn_cmd)



#!/usr/bin/python
#
# Copyright (c) 2013 CERN
# Author: Tomasz Wlostowski
#         Adrian Fiergolski
#
# This file is part of Hdlmake.
#
# Hdlmake is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Hdlmake is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Hdlmake.  If not, see <http://www.gnu.org/licenses/>.
#

# A Verilog preprocessor. Still lots of stuff to be done,
# but it's already quite useful for calculating dependencies.

"""This module provides the Verilog parser for HDLMake"""

from __future__ import print_function
from __future__ import absolute_import
import os
import re
import sys
import logging

from .new_dep_solver import DepParser
from .dep_file import DepRelation
from hdlmake.srcfile import create_source_file
import six


class VerilogPreprocessor(object):

    """This class provides the Verilog Preprocessor"""

    # Reserved verilog preprocessor keywords. The list is certainly not full
    vpp_keywords = [
        "default_nettype",
        "define",
        "line",
        "include",
        "elsif",
        "ifdef",
        "endif",
        "else",
        "undef",
        "timescale"]

    class VLDefine(object):

        """Class that provides a container for Verilog Defines"""

        def __init__(self, name, args, expansion):
            self.name = name
            self.args = args
            self.expansion = expansion

    class VLStack(object):

        """Class that provides a simple binary (true/false) stack
        for Verilog Defines for nested `ifdefs evaluation"""

        def __init__(self):
            self.stack = []

        def push(self, v_element):
            """Push element to the stack"""
            self.stack.append(v_element)

        def pop(self):
            "Pop element from the stack"""
            return self.stack.pop()

        def all_true(self):
            """Returns true if the stack is empty or all the contained
            elements are True"""
            return len(self.stack) == 0 or all(self.stack)

        def flip(self):
            """Toggle the following element"""
            self.push(not self.pop())

    def __init__(self):
        self.vpp_stack = self.VLStack()
        self.vlog_file = None
        # List of `include search paths
        self.vpp_searchdir = ["."]
        # List of macro definitions
        self.vpp_macros = []
        # Dictionary of files sub-included by each file parsed
        self.vpp_filedeps = {}

    def _find_macro(self, name):
        """Get the Verilog preprocessor macro named 'name'"""
        for macro_aux in self.vpp_macros:
            if macro_aux.name == name:
                return macro_aux
        return None

    def _search_include(self, filename, parent_dir=None):
        """Look for the 'filename' Verilog include file in the
        provided 'parent_dir'. If the directory is not provided, the method
        will search for the Verilog include in every defined Verilog
        preprocessor search directory"""
        if parent_dir is not None:
            possible_file = os.path.join(parent_dir, filename)
            if os.path.isfile(possible_file):
                return os.path.abspath(possible_file)
        for searchdir in self.vlog_file.include_dirs:
            probable_file = os.path.join(searchdir, filename)
            if os.path.isfile(probable_file):
                return os.path.abspath(probable_file)
        logging.error("Can't find %s for %s in any of the include "
                      "directories: %s", filename, self.vlog_file.file_path,
                      ', '.join(self.vlog_file.include_dirs))
        sys.exit("\nExiting")

    def _parse_macro_def(self, macro):
        """Parse the provided 'macro' and, if it's not a reserved keyword,
        create a new VLDefine instance and add it to the Verilog preprocessor
        list of macros"""
        name = macro.group(1)
        expansion = macro.group(3)
        if macro.group(2):
            params = macro.group(2).split(",")
        else:
            params = []
        if name in self.vpp_keywords:
            logging.error("Attempt to `define a reserved preprocessor keyword")
            quit(1)
        mdef = self.VLDefine(name, params, expansion)
        self.vpp_macros.append(mdef)
        return mdef

    def _preprocess_file(self, file_content, file_name, library):
        """Preprocess the content of the Verilog file"""
        def _remove_comment(text):
            """Function that removes the comments from the Verilog code"""
            def replacer(match):
                """Funtion that replace the matching comments"""
                text = match.group(0)
                if text.startswith('/'):
                    return ""
                else:
                    return text
            pattern = re.compile(
                r'//.*?$|/\*.*?\*/|"(?:\\.|[^\\"])*"',
                re.DOTALL | re.MULTILINE)
            return re.sub(pattern, replacer, text)

        def _degapize(text):
            """ Create a list in which the verilog sentences are
            stored in an ordered way -- and without empty 'gaps'"""
            lempty = re.compile(r"^\s*$")
            cline = None
            lines = []
            for line_aux in text.splitlines(False):
                if re.match(lempty, line_aux) is not None:
                    continue
                if line_aux.endswith('\\'):
                    if cline is None:
                        cline = ""
                    cline += line_aux[:len(line_aux) - 1]
                    continue
                elif cline:
                    line_aux = cline + line_aux
                    cline = None
                else:
                    cline = None
                lines.append(line_aux)
            return lines
        exps = {"include": re.compile(r"^\s*`include\s+\"(.+)\""),
                "define":
                re.compile(r"^\s*`define\s+(\w+)(?:\(([\w\s,]*)\))?(.*)"),
                "ifdef_elsif":
                re.compile(r"^\s*`(ifdef|ifndef|elsif)\s+(\w+)\s*$"),
                "endif_else": re.compile(r"^\s*`(endif|else)\s*$"),
                "begin_protected":
                re.compile(r"^\s*`pragma\s*protect\s*begin_protected\s*$"),
                "end_protected":
                re.compile(r"^\s*`pragma\s*protect\s*end_protected\s*$")}
        vl_macro_expand = re.compile(r"`(\w+)(?:\(([\w\s,]*)\))?")
        # init dependencies
        self.vpp_filedeps[file_name + library] = []
        cur_iter = 0
        logging.debug("preprocess file %s (of length %d) in library %s",
                      file_name, len(file_content), library)
        buf = _remove_comment(file_content)
        protected_region = False
        while True:
            new_buf = ""
            n_expansions = 0
            cur_iter += 1
            if cur_iter > 30:
                raise Exception("Recursion level exceeded. Nested `includes?")
            for line in _degapize(buf):
                matches = {}
                last = None
                for statement, stmt_regex in six.iteritems(exps):
                    matches[statement] = re.match(stmt_regex, line)
                    if matches[statement]:
                        last = matches[statement]
                if matches["begin_protected"]:
                    protected_region = True
                    continue
                if matches["end_protected"]:
                    protected_region = False
                    continue
                if protected_region:
                    continue
                if matches["ifdef_elsif"]:
                    cond_true = self._find_macro(last.group(2)) is not None
                    if last.group(1) == "ifndef":
                        cond_true = not cond_true
                    elif last.group(1) == "elsif":
                        self.vpp_stack.pop()
                    self.vpp_stack.push(cond_true)
                    continue
                elif matches["endif_else"]:
                    if last.group(1) == "endif":
                        self.vpp_stack.pop()
                    else:  # `else
                        self.vpp_stack.flip()
                    continue
                if not self.vpp_stack.all_true():
                    continue
                if matches["include"]:
                    included_file_path = self._search_include(
                        last.group(1), os.path.dirname(file_name))
                    logging.debug("File being parsed %s (library %s) "
                                  "includes %s",
                                  file_name, library, included_file_path)
                    line = self._preprocess_file(
                        file_content=open(included_file_path, "r").read(),
                        file_name=included_file_path, library=library)
                    self.vpp_filedeps[
                        file_name +
                        library].append(
                        included_file_path)
                    # add the whole include chain to the dependencies of the
                    # currently parsed file
                    self.vpp_filedeps[file_name + library].extend(
                        self.vpp_filedeps[included_file_path + library])
                    new_buf += line + '\n'
                    n_expansions += 1
                    continue
                elif matches["define"]:
                    self._parse_macro_def(matches["define"])

                def do_expand(what):
                    """Function to be applied by re.sub to every match of the
                    vl_macro_expand in the Verilof code -- group() returns
                    positive matches as indexed plain strings."""
                    if what.group(1) in self.vpp_keywords:
                        return '`' + what.group(1)
                    macro = self._find_macro(what.group(1))
                    if macro:
                        return macro.expansion
                    else:
                        logging.error("No expansion for macro '`%s' (%s) (%s)",
                                      what.group(1), line[:50]
                                      if len(line) > 50 else line, file_name)
                repl_line = re.sub(vl_macro_expand, do_expand, line)
                new_buf += repl_line + '\n'
                # if there was any expansion, then keep on iterating
                if repl_line != line:
                    n_expansions += 1
            buf = new_buf
            if n_expansions == 0:
                return new_buf

    def _define(self, name, expansion):
        """Define a new expansion Verilog macro and add it to the macro
        collection"""
        mdef = self.VLDefine(name, [], expansion)
        self.vpp_macros.append(mdef)

    def add_path(self, path):
        """Add a new path to the search directory list so that HDLMake
        will search for found includes on it"""
        self.vpp_searchdir.append(path)

    def preprocess(self, vlog_file):
        """Assign the provided 'vlog_file' to the associated class property
        and then preprocess and return the Verilog code"""
        # assert isinstance(vlog_file, VerilogFile)
        # assert isinstance(vlog_file, DepFile)
        self.vlog_file = vlog_file
        file_path = vlog_file.file_path
        buf = open(file_path, "r").read()
        return self._preprocess_file(file_content=buf,
                                     file_name=file_path,
                                     library=vlog_file.library)

    def get_file_deps(self):
        """Look for all of the defined preprocessor filedeps and return a list
        containing all of them"""
        deps = []
        for filedep_key in six.iterkeys(self.vpp_filedeps):
            for filedep in self.vpp_filedeps[filedep_key]:
                deps.append(filedep)
        return list(set(deps))


class VerilogParser(DepParser):

    """Class providing the Verilog Parser functionality"""

    reserved_words = ["accept_on",
                      "alias",
                      "always",
                      "always_comb",
                      "always_ff",
                      "always_latch",
                      "assert",
                      "assign",
                      "assume",
                      "automatic",
                      "before",
                      "begin",
                      "bind",
                      "bins",
                      "binsof",
                      "bit",
                      "break",
                      "buf",
                      "bufif0",
                      "bufif1",
                      "byte",
                      "case",
                      "casex",
                      "casez",
                      "cell",
                      "chandle",
                      "checker",
                      "class",
                      "clocking",
                      "cmos",
                      "config",
                      "const",
                      "constraint",
                      "context",
                      "continue",
                      "cover",
                      "covergroup",
                      "coverpoint",
                      "cross",
                      "deassign",
                      "default",
                      "defparam",
                      "disable",
                      "dist",
                      "do",
                      "edge",
                      "else",
                      "end",
                      "endcase",
                      "endchecker",
                      "endclass",
                      "endclocking",
                      "endconfig",
                      "endfunction",
                      "endgenerate",
                      "endgroup",
                      "endinterface",
                      "endmodule",
                      "endpackage",
                      "endprimitive",
                      "endprogram",
                      "endproperty",
                      "endsequence",
                      "endspecify",
                      "endtable",
                      "endtask",
                      "enum",
                      "event",
                      "eventually",
                      "expect",
                      "export",
                      "extends",
                      "extern",
                      "final",
                      "first_match",
                      "for",
                      "force",
                      "foreach",
                      "forever",
                      "fork",
                      "forkjoin",
                      "function",
                      "generate",
                      "genvar",
                      "global",
                      "highz0",
                      "highz1",
                      "if",
                      "iff",
                      "ifnone",
                      "ignore_bins",
                      "illegal_bins",
                      "implies",
                      "import",
                      "incdir",
                      "include",
                      "initial",
                      "inout",
                      "input",
                      "inside",
                      "instance",
                      "int",
                      "integer",
                      "interface",
                      "intersect",
                      "join",
                      "join_any",
                      "join_none",
                      "large",
                      "let",
                      "liblist",
                      "library",
                      "local",
                      "localparam",
                      "logic",
                      "longint",
                      "macromodule",
                      "matches",
                      "medium",
                      "modport",
                      "module",
                      "nand",
                      "negedge",
                      "new",
                      "nexttime",
                      "nmos",
                      "nor",
                      "noshowcancelled",
                      "not",
                      "notif0",
                      "notif1",
                      "null",
                      "or",
                      "output",
                      "package",
                      "packed",
                      "parameter",
                      "pmos",
                      "posedge",
                      "primitive",
                      "priority",
                      "program",
                      "property",
                      "protected",
                      "pull0",
                      "pull1",
                      "pulldown",
                      "pullup",
                      "pulsestyle_ondetect",
                      "pulsestyle_onevent",
                      "pure",
                      "rand",
                      "randc",
                      "randcase",
                      "randsequence",
                      "rcmos",
                      "real",
                      "realtime",
                      "ref",
                      "reg",
                      "reject_on",
                      "release",
                      "repeat",
                      "restrict",
                      "return",
                      "rnmos",
                      "rpmos",
                      "rtran",
                      "rtranif0",
                      "rtranif1",
                      "s_always",
                      "scalared",
                      "sequence",
                      "s_eventually",
                      "shortint",
                      "shortreal",
                      "showcancelled",
                      "signed",
                      "small",
                      "s_nexttime",
                      "solve",
                      "specify",
                      "specparam",
                      "static",
                      "string",
                      "strong",
                      "strong0",
                      "strong1",
                      "struct",
                      "s_until",
                      "super",
                      "supply0",
                      "supply1",
                      "sync_accept_on",
                      "sync_reject_on",
                      "table",
                      "tagged",
                      "task",
                      "this",
                      "throughout",
                      "time",
                      "timeprecision",
                      "timeunit",
                      "tran",
                      "tranif0",
                      "tranif1",
                      "tri",
                      "tri0",
                      "tri1",
                      "triand",
                      "trior",
                      "trireg",
                      "type",
                      "typedef",
                      "union",
                      "unique",
                      "unique0",
                      "unsigned",
                      "until",
                      "until_with",
                      "untypted",
                      "use",
                      "var",
                      "vectored",
                      "virtual",
                      "void",
                      "wait",
                      "wait_order",
                      "wand",
                      "weak",
                      "weak0",
                      "weak1",
                      "while",
                      "wildcard",
                      "wire",
                      "with",
                      "within",
                      "wor",
                      "xnor",
                      "xor"]

    def __init__(self, dep_file):
        DepParser.__init__(self, dep_file)
        self.preprocessor = VerilogPreprocessor()
        self.preprocessed = False

    def add_search_path(self, path):
        """Add a new candidate path to the Verilog preprocessor list
        containing the include dir candidates"""
        self.preprocessor.add_path(path)

    def parse(self, dep_file):
        """Parse the provided Verilog file and add to its properties
        all of the detected dependency relations"""
        if dep_file.is_parsed:
            return
        logging.debug("Parsing %s", dep_file.path)
        # assert isinstance(dep_file, DepFile), print("unexpected type: " +
        # str(type(dep_file)))
        buf = self.preprocessor.preprocess(dep_file)
        self.preprocessed = buf[:]
        # add includes as dependencies
        try:
            includes = self.preprocessor.vpp_filedeps[
                dep_file.path + dep_file.library]
            for file_aux in includes:
                dep_file.depends_on.add(
                    create_source_file(path=file_aux,
                                       module=dep_file.module,
                                       is_include=True))
            logging.debug("%s has %d includes.",
                          str(dep_file), len(includes))
        except KeyError:
            logging.debug(str(dep_file) + " has no includes.")
        # look for packages used inside in file
        # it may generate false dependencies as package in SV can be used by:
        #    import my_package::*;
        # or directly
        #    logic var = my_package::MY_CONST;
        # The same way constants and others can be imported directly from
        # other modules:
        #    logic var = my_other_module::MY_CONST;
        # and HdlMake will anyway create dependency marking my_other_module as
        # requested package
        import_pattern = re.compile(r"(\w+) *::(\w+|\\*)")

        def do_imports(text):
            """Function to be applied by re.subn to every match of the
            import_pattern in the Verilog code -- group() returns positive
            matches as indexed plain strings. It adds the found USE
            relations to the file"""
            logging.debug("file %s imports/uses %s.%s package",
                          dep_file.path, dep_file.library, text.group(1))
            dep_file.add_relation(
                DepRelation("%s.%s" % (dep_file.library, text.group(1)),
                            DepRelation.USE, DepRelation.PACKAGE))
        re.subn(import_pattern, do_imports, buf)
        # packages
        m_inside_package = re.compile(
            r"package\s+(\w+)\s*(?:\(.*?\))?\s*(.+?)endpackage",
            re.DOTALL | re.MULTILINE)

        def do_package(text):
            """Function to be applied by re.subn to every match of the
            m_inside_pattern in the Verilog code -- group() returns positive
            matches as indexed plain strings. It adds the found PROVIDE
            relations to the file"""
            logging.debug("found pacakge %s.%s", dep_file.library,
                          text.group(1))
            dep_file.add_relation(
                DepRelation("%s.%s" % (dep_file.library, text.group(1)),
                            DepRelation.PROVIDE, DepRelation.PACKAGE))
        re.subn(m_inside_package, do_package, buf)
        # modules and instatniations
        m_inside_module = re.compile(
            r"(?:module|interface)\s+(\w+)\s*(?:\(.*?\))?\s*(.+?)"
            r"(?:endmodule|endinterface)",
            re.DOTALL | re.MULTILINE)
        m_instantiation = re.compile(
            r"(?:\A|\s*)\s*(\w+)\s+(?:#\s*\(.*?\)\s*)?(\w+)\s*\(.*?\)\s*",
            re.DOTALL | re.MULTILINE)

        def do_module(text):
            """Function to be applied by re.sub to every match of the
            m_inside_module in the Verilog code -- group() returns
            positive  matches as indexed plain strings. It adds the found
            PROVIDE relations to the file"""
            logging.debug("found module %s.%s", dep_file.library,
                          text.group(1))
            dep_file.add_relation(
                DepRelation("%s.%s" % (dep_file.library, text.group(1)),
                            DepRelation.PROVIDE, DepRelation.MODULE))

            def do_inst(text):
                """Function to be applied by re.sub to every match of the
                m_instantiation in the Verilog code -- group() returns positive
                matches as indexed plain strings. It adds the found USE
                relations to the file"""
                mod_name = text.group(1)
                if mod_name in self.reserved_words:
                    return
                logging.debug("-> instantiates %s.%s as %s",
                              dep_file.library, text.group(1), text.group(2))
                dep_file.add_relation(
                    DepRelation("%s.%s" % (dep_file.library, text.group(1)),
                                DepRelation.USE, DepRelation.MODULE))
            re.subn(m_instantiation, do_inst, text.group(2))
        re.subn(m_inside_module, do_module, buf)
        dep_file.add_relation(
            DepRelation(
                dep_file.path,
                DepRelation.PROVIDE,
                DepRelation.INCLUDE))
        dep_file.is_parsed = True



#!/usr/bin/python
#
# Copyright (c) 2013, 2014 CERN
# Author: Pawel Szostek (pawel.szostek@cern.ch)
# Multi-tool support by Javier D. Garcia-Lasheras (javier@garcialasheras.com)
#
# This file is part of Hdlmake.
#
# Hdlmake is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Hdlmake is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Hdlmake.  If not, see <http://www.gnu.org/licenses/>.
#

"""This package provides the functions and classes to parse and solve
 HDLMake filesets"""

from __future__ import print_function
from __future__ import absolute_import
import logging

from .dep_file import DepFile


class DepParser(object):

    """Base Class for the different HDL parsers (VHDL and Verilog)"""

    def __init__(self, dep_file):
        self.dep_file = dep_file

    def parse(self, dep_file):
        """Base dummy interface method for the HDL parse execution"""
        pass


def solve(fileset, standard_libs=None):
    """Function that Parses and Solves the provided HDL fileset. Note
       that it doesn't return a new fileset, but modifies the original one"""
    from .srcfile import SourceFileSet
    from .dep_file import DepRelation
    assert isinstance(fileset, SourceFileSet)
    fset = fileset.filter(DepFile)
    # print(fileset)
    # print(fset)
    not_satisfied = 0
    logging.debug("PARSE BEGIN: Here, we will parse all the files in the "
                  "fileset: no parsing should be done beyond this point")
    for investigated_file in fset:
        logging.debug("INVESTIGATED FILE: %s", investigated_file)
        if not investigated_file.is_parsed:
            logging.debug("Not parsed yet, let's go!")
            investigated_file.parser.parse(investigated_file)
    logging.debug("PARSE END: now the parsing is done")
    logging.debug("SOLVE BEGIN")
    for investigated_file in fset:
        # logging.info("INVESTIGATED FILE: %s" % investigated_file)
        # print(investigated_file.rels)
        for rel in investigated_file.rels:
            # logging.info("- relation: %s" % rel)
            # logging.info("- direction: %s" % rel.direction)
            # Only analyze USE relations, we are looking for dependencies
            if rel.direction == DepRelation.USE:
                satisfied_by = set()
                for dep_file in fset:
                    if dep_file.satisfies(rel):
                        if dep_file is not investigated_file:
                            investigated_file.depends_on.add(dep_file)
                        satisfied_by.add(dep_file)
                if len(satisfied_by) > 1:
                    logging.warning(
                        "Relation %s satisfied by multpiple (%d) files: %s",
                        str(rel),
                        len(satisfied_by),
                        '\n'.join([file_aux.path for
                                   file_aux in list(satisfied_by)]))
                elif len(satisfied_by) == 0:
                    # if relation is a USE PACKAGE, check against
                    # the standard libs provided by the tool HDL compiler
                    required_lib = rel.obj_name.split('.')[0]
                    if (not standard_libs is None and
                        required_lib in standard_libs and
                        rel.direction is DepRelation.USE and
                            rel.rel_type is DepRelation.PACKAGE):
                        logging.debug("Not satisfied relation %s in %s will "
                                      "be covered by the target compiler "
                                      "standard libs.",
                                      str(rel), investigated_file.name)
                    else:
                        logging.warning("Relation %s in %s not satisfied by "
                                        "any source file",
                                        str(rel), investigated_file.name)
                        not_satisfied += 1
    logging.debug("SOLVE END")
    if not_satisfied != 0:
        logging.warning(
            "Dependencies solved, but %d relations were not satisfied",
            not_satisfied)
    else:
        logging.info(
            "Dependencies solved, all of the relations were satisfied!")


def make_dependency_sorted_list(fileset, reverse=False):
    """Sort files in order of dependency.
    Files with no dependencies first.
    All files that another depends on will be earlier in the list."""
    dependable = [f for f in fileset if isinstance(f, DepFile)]
    non_dependable = [f for f in fileset if not isinstance(f, DepFile)]
    dependable.sort(key=lambda f: f.file_path.lower())
    # Not necessary, but will tend to group files more nicely
    # in the output.
    dependable.sort(key=DepFile.get_dep_level)
    sorted_list = non_dependable + dependable
    if reverse:
        sorted_list = list(reversed(sorted_list))
    return sorted_list


def make_dependency_set(fileset, top_level_entity, extra_modules=None):
    """Create the set of all files required to build the named
     top_level_entity."""
    from hdlmake.srcfile import SourceFileSet
    from hdlmake.dep_file import DepRelation
    assert isinstance(fileset, SourceFileSet)
    fset = fileset.filter(DepFile)

    def _check_entity(test_file, entity_name):
        """ Check if the input file provides the entity pointed by the name"""
        if entity_name == None:
            return False
        entity_rel_vhdl = DepRelation(
            entity_name,
            DepRelation.PROVIDE, DepRelation.ENTITY)
        entity_rel_vlog = DepRelation(
            "%s.%s" %
            ("work", entity_name), DepRelation.PROVIDE, DepRelation.MODULE)
        for rel in test_file.rels:
            if (rel == entity_rel_vhdl) or (rel == entity_rel_vlog):
                return True
        return False

    top_file = None
    extra_files = []
    for chk_file in fset:
        if _check_entity(chk_file, top_level_entity):
            top_file = chk_file
        if not extra_modules == None:
            for entity_aux in extra_modules:
                if _check_entity(chk_file, entity_aux):
                    extra_files.append(chk_file)
    if top_file is None:
        if top_level_entity is None:
            logging.critical(
                    'Could not find a top level file because the top '
                    'module is undefined. Continuing with the full file set.')
        else:
            logging.critical(
                    'Could not find a top level file that provides the '
                    'top_module="%s". Continuing with the full file set.',
                     top_level_entity)
        return fileset
    # Collect only the files that the top level entity is dependant on, by
    # walking the dependancy tree.
    try:
        dep_file_set = set()
        file_set = set([top_file] + extra_files)
        while True:
            chk_file = file_set.pop()
            dep_file_set.add(chk_file)
            file_set.update(chk_file.depends_on - dep_file_set)
    except KeyError:
        # no files left
        pass
    hierarchy_drivers = [top_level_entity]
    if not extra_modules == None:
        hierarchy_drivers += extra_modules
    logging.info("Found %d files as dependancies of %s.",
                 len(dep_file_set), ", ".join(hierarchy_drivers))
    return dep_file_set



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2013 - 2015 CERN
# Author: Pawel Szostek (pawel.szostek@cern.ch)
# Multi-tool support by Javier D. Garcia-Lasheras (javier@garcialasheras.com)
#
# This file is part of Hdlmake.
#
# Hdlmake is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Hdlmake is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Hdlmake.  If not, see <http://www.gnu.org/licenses/>.
#

"""Module providing support for Xilinx Vivado simulation"""


from __future__ import absolute_import
from .make_sim import ToolSim
from hdlmake.srcfile import VerilogFile, VHDLFile, SVFile

class ToolVivadoSim(ToolSim):

    """Class providing the interface for Xilinx Vivado synthesis"""

    TOOL_INFO = {
        'name': 'vivado-sim',
        'id': 'vivado-sim',
        'windows_bin': 'vivado -mode tcl -source',
        'linux_bin': 'vivado -mode tcl -source',
    }

    STANDARD_LIBS = ['ieee', 'std']

    HDL_FILES = {VerilogFile: '', VHDLFile: '', SVFile: ''}

    CLEAN_TARGETS = {'clean': [".Xil", "*.jou", "*.log", "*.pb",
                               "work", "xsim.dir"],
                     'mrproper': ["*.wdb", "*.vcd"]}

    SIMULATOR_CONTROLS = {'vlog': 'xvlog $<',
                          'vhdl': 'xvhdl $<',
                          'compiler': 'xelab -debug all $(TOP_MODULE) '
                                      '-s $(TOP_MODULE)'}

    def __init__(self):
        super(ToolVivadoSim, self).__init__()
        self._tool_info.update(ToolVivadoSim.TOOL_INFO)
        self._standard_libs.extend(ToolVivadoSim.STANDARD_LIBS)
        self._clean_targets.update(ToolVivadoSim.CLEAN_TARGETS)
        self._simulator_controls.update(ToolVivadoSim.SIMULATOR_CONTROLS)
        self._hdl_files.update(ToolVivadoSim.HDL_FILES)

    def _makefile_sim_compilation(self):
        """Generate compile simulation Makefile target for Vivado Simulator"""
        self.writeln("simulation: $(VERILOG_OBJ) $(VHDL_OBJ)")
        self.writeln("\t\t" + ToolVivadoSim.SIMULATOR_CONTROLS['compiler'])
        self.writeln()
        self._makefile_sim_dep_files()



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2013 - 2015 CERN
# Author: Pawel Szostek (pawel.szostek@cern.ch)
# Multi-tool support by Javier D. Garcia-Lasheras (javier@garcialasheras.com)
#
# This file is part of Hdlmake.
#
# Hdlmake is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Hdlmake is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Hdlmake.  If not, see <http://www.gnu.org/licenses/>.
#

"""Module providing support for Mentor Modelsim simulation"""

from __future__ import print_function
from __future__ import absolute_import
import os

from .sim_makefile_support import VsimMakefileWriter


class ToolModelsim(VsimMakefileWriter):

    """Class providing the interface for Mentor Modelsim simulator"""

    TOOL_INFO = {'name': 'Modelsim',
                 'id': 'modelsim',
                 'windows_bin': 'vsim.exe',
                 'linux_bin': 'vsim'}

    STANDARD_LIBS = ['ieee', 'std', 'altera_mf']

    CLEAN_TARGETS = {'clean': ["modelsim.ini", "transcript"],
                     'mrproper': ["*.vcd", "*.wlf"]}

    def __init__(self):
        super(ToolModelsim, self).__init__()
        self.copy_rules["modelsim.ini"] = os.path.join(
            "$(MODELSIM_INI_PATH)", "modelsim.ini")
        self.additional_deps.append("modelsim.ini")
        self._tool_info.update(ToolModelsim.TOOL_INFO)
        self._clean_targets.update(ToolModelsim.CLEAN_TARGETS)
        self._standard_libs.extend(ToolModelsim.STANDARD_LIBS)

    def _makefile_sim_options(self):
        """Print the Modelsim options to the Makefile"""
        modelsim_ini_path = self.manifest_dict.get("modelsim_ini_path")
        if modelsim_ini_path == None:
            if "sim_path" in self.manifest_dict:
                modelsim_ini_path = os.path.join(
                    self.manifest_dict["sim_path"], "..")
            else:
                modelsim_ini_path = os.path.join(
                    "$(HDLMAKE_MODELSIM_PATH)", "..")
        self.custom_variables["MODELSIM_INI_PATH"] = modelsim_ini_path
        modelsim_ini = "-modelsimini modelsim.ini "
        vcom_opt = self.manifest_dict.get("vcom_opt", '')
        self.manifest_dict["vcom_opt"] = modelsim_ini + vcom_opt
        vlog_opt = self.manifest_dict.get("vlog_opt", '')
        self.manifest_dict["vlog_opt"] = modelsim_ini + vlog_opt
        vmap_opt = self.manifest_dict.get("vmap_opt", '')
        self.manifest_dict["vmap_opt"] = modelsim_ini + vmap_opt
        super(ToolModelsim, self)._makefile_sim_options()



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2013 CERN
# Author: Pawel Szostek (pawel.szostek@cern.ch)
#
# This file is part of Hdlmake.
#
# Hdlmake is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Hdlmake is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Hdlmake.  If not, see <http://www.gnu.org/licenses/>.

"""This module provides the core actions to the pool"""

from __future__ import absolute_import
from __future__ import print_function
import logging
import os
import sys
import os.path

import hdlmake.fetch as fetch
import hdlmake.new_dep_solver as dep_solver
from hdlmake.util import path as path_mod
from hdlmake.fetch import Svn, Git, GitSM, Local
from hdlmake.fetch import SVN, GIT, GITSM, LOCAL
from .action import Action


class ActionCore(Action):

    """Class that contains the methods for core actions"""

    def __init__(self, *args):
        super(ActionCore, self).__init__(*args)
        self.git_backend = Git()
        self.gitsm_backend = GitSM()
        self.svn_backend = Svn()
        self.local_backend = Local()

    def _check_all_fetched(self):
        """Check if every module in the pool is fetched"""

        if not len([m for m in self if not m.isfetched]) == 0:
            logging.error(
                "Fetching should be done before continuing.\n"
                "The following modules remains unfetched:\n"
                "%s",
                "\n".join([str(m) for m in self if not m.isfetched]))
            quit(1)
    def makefile(self):
        """Write the Makefile for the current design"""
        self._check_all_fetched()
        self.build_file_set()
        self.solve_file_set()
        combined_fileset = self.parseable_fileset
        combined_fileset.add(self.privative_fileset)
        self.tool.write_makefile(self.config,
                                 combined_fileset,
                                 filename=self.options.filename)

    def _fetch_all(self):
        """Fetch all the modules declared in the design"""

        def _fetch_module(module):
            """Fetch the given module from the remote origin"""
            new_modules = []
            logging.debug("Fetching module: %s", str(module))
            if module.source is SVN:
                result = self.svn_backend.fetch(module)
            elif module.source is GIT:
                result = self.git_backend.fetch(module)
            elif module.source is GITSM:
                result = self.gitsm_backend.fetch(module)
            elif module.source is LOCAL:
                result = self.local_backend.fetch(module)
            if result is False:
                logging.error("Unable to fetch module %s", str(module.url))
                sys.exit("Exiting")
            module.parse_manifest()
            new_modules.extend(module.local)
            new_modules.extend(module.svn)
            new_modules.extend(module.git)
            new_modules.extend(module.gitsm)
            return new_modules

        fetch_queue = [m for m in self]

        while len(fetch_queue) > 0:
            cur_mod = fetch_queue.pop()
            new_modules = []
            if cur_mod.isfetched:
                new_modules = cur_mod.submodules()
            else:
                new_modules = _fetch_module(cur_mod)
            for mod in new_modules:
                if not mod.isfetched:
                    logging.debug("Appended to fetch queue: "
                                  + str(mod.url))
                    self._add(mod)
                    fetch_queue.append(mod)
                else:
                    logging.debug("NOT appended to fetch queue: "
                                  + str(mod.url))

    def fetch(self):
        """Fetch the missing required modules from their remote origin"""
        logging.info("Fetching needed modules.")
        for mod in self:
            if mod.isfetched and not mod.manifest_dict == None:
                if 'fetch_pre_cmd' in mod.manifest_dict:
                    os.system(mod.manifest_dict.get("fetch_pre_cmd", ''))
        self._fetch_all()
        for mod in self:
            if mod.isfetched and not mod.manifest_dict == None:
                if 'fetch_post_cmd' in mod.manifest_dict:
                    os.system(mod.manifest_dict.get("fetch_post_cmd", ''))
        logging.info("All modules fetched.")

    def clean(self):
        """Delete the local copy of the fetched modules"""
        logging.info("Removing fetched modules..")
        remove_list = [mod_aux for mod_aux in self
                       if mod_aux.source in [fetch.GIT, fetch.GITSM, fetch.SVN]
                       and mod_aux.isfetched]
        remove_list.reverse()  # we will remove modules in backward order
        if len(remove_list):
            for mod_aux in remove_list:
                logging.info("... clean: " + mod_aux.url +
                             " [from: " + mod_aux.path + "]")
                mod_aux.remove_dir_from_disk()
        else:
            logging.info("There are no modules to be removed")
        logging.info("Modules cleaned.")

    def list_files(self):
        """List the files added to the design across the pool hierarchy"""
        unfetched_modules = [mod_aux for mod_aux in self
                             if not mod_aux.isfetched]
        for mod_aux in unfetched_modules:
            logging.warning(
                "List incomplete, module %s has not been fetched!", mod_aux)
        if self.options.top != None:
            self.top_entity = self.options.top
        self.build_file_set()
        self.solve_file_set()
        file_list = dep_solver.make_dependency_sorted_list(
            self.parseable_fileset)
        files_str = [file_aux.path for file_aux in file_list]
        if self.options.reverse is True:
            files_str.reverse()
        if self.options.delimiter is None:
            delimiter = "\n"
        else:
            delimiter = self.options.delimiter
        print(delimiter.join(files_str))

    def _print_comment(self, message):
        """Private method that prints a message to stdout if not terse"""
        if not self.options.terse:
            print(message)

    def _print_file_list(self, file_list):
        """Print file list to standard out"""
        if not len(file_list):
            self._print_comment("# * This module has no files")
        else:
            for file_aux in file_list:
                print("%s\t%s" % (
                    path_mod.relpath(file_aux.path), "file"))

    def list_modules(self):
        """List the modules that are contained by the pool"""

        def _convert_to_source_name(source_code):
            """Private function that returns a string with the source type"""
            if source_code == fetch.GIT:
                return "git"
            elif source_code == fetch.GITSM:
                return "gitsm"
            elif source_code == fetch.SVN:
                return "svn"
            elif source_code == fetch.LOCAL:
                return "local"

        for mod_aux in self:
            if not mod_aux.isfetched:
                logging.warning("Module not fetched: %s", mod_aux.url)
                self._print_comment("# MODULE UNFETCHED! -> %s" % mod_aux.url)
            else:
                self._print_comment("# MODULE START -> %s" % mod_aux.url)
                if mod_aux.source in [fetch.SVN, fetch.GIT, fetch.GITSM]:
                    self._print_comment("# * URL: " + mod_aux.url)
                if (mod_aux.source
                        in [fetch.SVN, fetch.GIT, fetch.GITSM, fetch.LOCAL]
                        and mod_aux.parent):
                    self._print_comment("# * The parent for this module is: %s"
                                        % mod_aux.parent.url)
                else:
                    self._print_comment("# * This is the root module")
                print("%s\t%s" % (mod_aux.path,
                                  _convert_to_source_name(mod_aux.source)))
                if self.options.withfiles:
                    self._print_file_list(mod_aux.files)
                self._print_comment("# MODULE END -> %s" % mod_aux.url)
            self._print_comment("")



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2013, 2014 CERN
# Author: Pawel Szostek (pawel.szostek@cern.ch)
# Multi-tool support by Javier D. Garcia-Lasheras (javier@garcialasheras.com)
#
# This file is part of Hdlmake.
#
# Hdlmake is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Hdlmake is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Hdlmake.  If not, see <http://www.gnu.org/licenses/>.

"""The Action package provides the full set of provided user functionalities"""

from .core import ActionCore
from .tree import ActionTree



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2013-2015 CERN
# Author:
#     Tomasz Wlostowski (tomasz.wlostowski@cern.ch)
#     Adrian Fiergolski (Adrian.Fiergolski@cern.ch)
#
# This file is part of Hdlmake.
#
# Hdlmake is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Hdlmake is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Hdlmake.  If not, see <http://www.gnu.org/licenses/>.

"""Module providing the VHDL parser capabilities"""

from __future__ import absolute_import
import logging
import re

from .new_dep_solver import DepParser


class VHDLParser(DepParser):

    """Class providing the container for VHDL parser instances"""

    def __init__(self, dep_file):
        DepParser.__init__(self, dep_file)
        # self.preprocessor = VHDLPreprocessor()

    def parse(self, dep_file):
        """Parse the provided VHDL file and add the detected relations to it"""
        from .dep_file import DepRelation
        if dep_file.is_parsed:
            return
        logging.debug("Parsing %s", dep_file.path)

        def _preprocess(vhdl_file):
            """Preprocess the supplied VHDL file instance"""
            def _preprocess_file(file_content, file_name, library):
                """Preprocess the suplied string using the arguments"""
                def _remove_comments_and_strings(text):
                    """Remove the comments and strings from the VHDL code"""
                    pattern = re.compile('--.*?$|".?"',
                                         re.DOTALL | re.MULTILINE)
                    return re.sub(pattern, "", text)
                logging.debug(
                    "preprocess file %s (of length %d) in library %s",
                    file_name, len(file_content), library)
                return _remove_comments_and_strings(file_content)
            file_path = vhdl_file.file_path
            buf = open(file_path, "r").read()
            return _preprocess_file(file_content=buf,
                                    file_name=file_path,
                                    library=vhdl_file.library)
        buf = _preprocess(dep_file)
        # use packages
        use_pattern = re.compile(
            r"^\s*use\s+(\w+)\s*\.\s*(\w+)",
            re.DOTALL | re.MULTILINE | re.IGNORECASE)

        def do_use(text):
            """Function to be applied by re.sub to every match of the
            use_pattern in the VHDL code -- group() returns positive matches
            as indexed plain strings. It adds the found USE relations to the
            file"""
            if text.group(1).lower() == "work":
                logging.debug("use package %s.%s",
                              dep_file.library, text.group(2))
                dep_file.add_relation(
                    DepRelation("%s.%s" % (dep_file.library, text.group(2)),
                                DepRelation.USE,
                                DepRelation.PACKAGE))
            else:
                logging.debug("use package %s.%s",
                              text.group(1), text.group(2))
                dep_file.add_relation(
                    DepRelation("%s.%s" % (text.group(1), text.group(2)),
                                DepRelation.USE,
                                DepRelation.PACKAGE))
            return "<hdlmake use_pattern %s.%s>" % (text.group(1),
                                                    text.group(2))
        buf = re.sub(use_pattern, do_use, buf)
        # new entity
        entity_pattern = re.compile(
            r"^\s*entity\s+(?P<name>\w+)\s+is\s+(?:port|generic|end)"
            r".*?((?P=name)|entity)\s*;",
            re.DOTALL | re.MULTILINE | re.IGNORECASE)
        def do_entity(text):
            """Function to be applied by re.sub to every match of the
            entity_pattern in the VHDL code -- group() returns positive matches
            as indexed plain strings. It adds the found PROVIDE relations
            to the file"""
            logging.debug("found entity %s.%s",
                          dep_file.library, text.group(1))
            dep_file.add_relation(
                DepRelation("%s.%s" % (dep_file.library, text.group(1)),
                            DepRelation.PROVIDE,
                            DepRelation.ENTITY))
            return "<hdlmake entity_pattern %s.%s>" % (dep_file.library,
                                                       text.group(1))

        buf = re.sub(entity_pattern, do_entity, buf)

        # new architecture
        architecture_pattern = re.compile(
            r"^\s*architecture\s+(?P<name>\w+)\s+of\s+(\w+)\s+is"
            r".*end\s*((|architecture)\s*(?P=name)|architecture)\s*;",
            re.DOTALL | re.MULTILINE | re.IGNORECASE)
        architecture_split_pattern = re.compile(
            r"^\s*architecture\s+(?P<name>\w+)\s+of\s+(\w+)\s+is",
            re.DOTALL | re.MULTILINE | re.IGNORECASE)

        def do_architecture(text):
            """Function to be applied by re.sub to every match of the
            architecture_pattern in the VHDL code -- group() returns positive
            matches as indexed plain strings. It adds the found PROVIDE
            relations to the file"""
            logging.debug("found architecture %s of entity %s.%s",
                          text.group(1), dep_file.library, text.group(2))
            dep_file.add_relation(
                DepRelation("%s.%s" % (dep_file.library, text.group(2)),
                            DepRelation.PROVIDE,
                            DepRelation.ARCHITECTURE))
            dep_file.add_relation(
                DepRelation("%s.%s" % (dep_file.library, text.group(2)),
                            DepRelation.USE,
                            DepRelation.ENTITY))

            return "<hdlmake architecture %s.%s>" % (dep_file.library,
                                                     text.group(2))
        buf = re.sub(architecture_split_pattern, do_architecture, buf)

        # new package
        package_pattern = re.compile(
            r"^\s*package\s+(\w+)\s+is",
            re.DOTALL | re.MULTILINE | re.IGNORECASE)

        def do_package(text):
            """Function to be applied by re.sub to every match of the
            package_pattern in the VHDL code -- group() returns positive
            matches as indexed plain strings. It adds the found PROVIDE
            relations to the file"""
            logging.debug("found package %s.%s", dep_file.library,
                          text.group(1))
            dep_file.add_relation(
                DepRelation("%s.%s" % (dep_file.library, text.group(1)),
                            DepRelation.PROVIDE,
                            DepRelation.PACKAGE))
            return "<hdlmake package %s.%s>" % (dep_file.library,
                                                text.group(1))
        buf = re.sub(package_pattern, do_package, buf)

        # component declaration
        component_pattern = re.compile(
            r"^\s*component\s+(\w+).*?end\s+component.*?;",
            re.DOTALL | re.MULTILINE | re.IGNORECASE)

        def do_component(text):
            """Function to be applied by re.sub to every match of the
            component_pattern in the VHDL code -- group() returns positive
            matches as indexed plain strings. It doesn't add any relation
            to the file"""
            logging.debug("found component declaration %s", text.group(1))
            #dep_file.add_relation(
            #    DepRelation("%s.%s" % (dep_file.library, text.group(1)),
            #                DepRelation.USE,
            #                DepRelation.ENTITY))
            return "<hdlmake component %s>" % text.group(1)

        buf = re.sub(component_pattern, do_component, buf)

        # Signal declaration
        signal_pattern = re.compile(
            r"^\s*signal\s+(\w+).*?;",
            re.DOTALL | re.MULTILINE | re.IGNORECASE)

        def do_signal(text):
            """Function to be applied by re.sub to every match of the
            signal_pattern in the VHDL code -- group() returns positive
            matches as indexed plain strings. It doesn't add any relation
            to the file"""
            logging.debug("found signal declaration %s", text.group(1))
            return "<hdlmake signal %s>" % text.group(1)

        buf = re.sub(signal_pattern, do_signal, buf)

        # Constant declaration
        constant_pattern = re.compile(
            r"^\s*constant\s+(\w+).*?;",
            re.DOTALL | re.MULTILINE | re.IGNORECASE)

        def do_constant(text):
            """Function to be applied by re.sub to every match of the
            constant_pattern in the VHDL code -- group() returns positive
            matches as indexed plain strings. It doesn't add any relation
            to the file"""
            logging.debug("found constant declaration %s", text.group(1))
            return "<hdlmake constant %s>" % text.group(1)

        buf = re.sub(constant_pattern, do_constant, buf)


        # record declaration
        record_pattern = re.compile(
            r"^\s*type\s+(\w+)\s+is\s+record.*?end\s+record.*?;",
            re.DOTALL | re.MULTILINE | re.IGNORECASE)

        def do_record(text):
            """Function to be applied by re.sub to every match of the
            record_pattern in the VHDL code -- group() returns positive matches
            as indexed plain strings. It doesn't add any relation to the
            file"""
            logging.debug("found record declaration %s", text.group(1))
            return "<hdlmake record %s>" % text.group(1)

        buf = re.sub(record_pattern, do_record, buf)

        # function declaration
        function_pattern = re.compile(
            r"^\s*function\s+(?P<name>\w+)"
            r".*?" # gobble arguments if any.
            r"return\s+\w+"
            r"(\s+is.*?end\s+function.*?)?" # gobble body if any.
            r"\s*;",
            re.DOTALL | re.MULTILINE | re.IGNORECASE)

        def do_function(text):
            """Function to be applied by re.sub to every match of the
            funtion_pattern in the VHDL code -- group() returns positive
            matches as indexed plain strings. It doesn't add the relations
            to the file"""
            logging.debug("found function declaration %s", text.group(1))
            return "<hdlmake function %s>" % text.group(1)

        buf = re.sub(function_pattern, do_function, buf)

        # instantions
        libraries = set([dep_file.library])
        instance_pattern = re.compile(
            r"^\s*(?P<LABEL>\w+)\s*:"
            r"\s*(?:entity\s+(?P<LIB>\w+)\.)?(?P<ENTITY>\w+)"
            r"\s*(?:\(\s*(?P<ARCH>\w+)\s*\)\s*)?"
            r"(?:port\s+map.*?|generic\s+map.*?)",
            re.DOTALL | re.MULTILINE | re.IGNORECASE)

        def do_instance(text):
            """Function to be applied by re.sub to every match of the
            instance_pattern in the VHDL code -- group() returns positive
            matches as indexed plain strings. It adds the found USE
            relations to the file"""
            logging.debug("-> instantiates %s.%s(%s) as %s",
                          text.group("LIB"), text.group("ENTITY"), text.group("ARCH"), text.group("LABEL"))
            lib = text.group("LIB")
            if not lib or lib == "work":
                lib = dep_file.library
            dep_file.add_relation(DepRelation(
                "%s.%s" % (lib, text.group("ENTITY")),
                DepRelation.USE, DepRelation.ENTITY))
            return "<hdlmake instance %s|%s|%s>" % (text.group("LABEL"),
                                                    lib,
                                                    text.group("ENTITY"))
        buf = re.sub(instance_pattern, do_instance, buf)

        instance_from_library_pattern = re.compile(
            r"^\s*(\w+)\s*\:\s*entity\s*(\w+)\s*\.\s*(\w+)\s*(?:port"
            r"\s+map.*?;|generic\s+map.*?;|\s*;)",
            re.DOTALL | re.MULTILINE | re.IGNORECASE)

        def do_instance_from_library(text):
            """Function to be applied by re.sub to every match of the
            instance_from_library_pattern in the VHDL code -- group()
            returns positive matches as indexed plain strings.
            It adds the found USE relations to the file"""
            if text.group(2).lower() == "work":
                logging.debug("-> instantiates %s.%s as %s",
                              dep_file.library, text.group(3), text.group(1))
                #dep_file.add_relation(
                #    DepRelation("%s.%s" % (dep_file.library, text.group(3)),
                #                DepRelation.USE,
                #                DepRelation.ARCHITECTURE))
            else:
                logging.debug("-> instantiates %s.%s as %s",
                              text.group(2), text.group(3), text.group(1))
                #dep_file.add_relation(
                #    DepRelation("%s.%s" % (text.group(2), text.group(3)),
                #                DepRelation.USE,
                #                DepRelation.ARCHITECTURE))
            return "<hdlmake instance_from_library %s|%s>" % (text.group(1),
                                                              text.group(3))
        buf = re.sub(instance_from_library_pattern,
                     do_instance_from_library, buf)
        # libraries
        library_pattern = re.compile(
            r"^\s*library\s*(\w+)\s*;",
            re.DOTALL | re.MULTILINE | re.IGNORECASE)

        def do_library(text):
            """Function to be applied by re.sub to every match of the
            library_pattern in the VHDL code -- group() returns positive
            matches as indexed plain strings. It adds the used libraries
            to the file's 'library' property"""
            logging.debug("use library %s", text.group(1))
            libraries.add(text.group(1))
            return "<hdlmake library %s>" % text.group(1)
        buf = re.sub(library_pattern, do_library, buf)
        # logging.debug("\n" + buf) # print modified buffer.
        dep_file.is_parsed = True



action = "simulation"
sim_tool = "ghdl"
sim_top = "counter_tb"

sim_post_cmd = "ghdl -r counter_tb --stop-time=6us --vcd=counter_tb.vcd; gtkwave counter_tb.vcd"

modules = {
  "local" : [ "../../../testbench/counter_tb/vhdl" ],
}



target = "microsemi"
action = "synthesis"

syn_device = "a3p250"
syn_grade = "-2"
syn_package = "208 pqfp"
syn_top = "proasic3_top"
syn_project = "demo"
syn_tool = "libero"

modules = {
  "local" : [ "../../../top/proasic3_sk/vhdl" ],
}





# -*- coding: utf-8 -*-
#
# Copyright (c) 2013, 2014 CERN
# Author: Pawel Szostek (pawel.szostek@cern.ch)
# Multi-tool support by Javier D. Garcia-Lasheras (javier@garcialasheras.com)
#
# This file is part of Hdlmake.
#
# Hdlmake is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Hdlmake is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Hdlmake.  If not, see <http://www.gnu.org/licenses/>.
#

"""Module providing the source file class and a set of classes
representing the different possible files and file extensions"""

from __future__ import print_function
from __future__ import absolute_import
import os
import logging

from .util import path as path_mod
from .dep_file import DepFile, File
import six


class SourceFile(DepFile):

    """This is a class acting as a base for the different
    HDL sources files, i.e. those that can be parsed"""

    cur_index = 0

    def __init__(self, path, module, library):
        assert isinstance(path, six.string_types)
        self.is_include = False
        self.library = library
        if not library:
            self.library = "work"
        DepFile.__init__(self,
                         file_path=path,
                         module=module)

    def __hash__(self):
        return hash(self.path + self.library)


# SOURCE FILES

class VHDLFile(SourceFile):

    """This is the class providing the generic VHDL file"""

    def __init__(self, path, module, library=None):
        SourceFile.__init__(self, path=path, module=module, library=library)
        from hdlmake.vhdl_parser import VHDLParser
        self.parser = VHDLParser(self)

    def _check_encryption(self):
        """Check if the VHDL is encrypted (in Xilinx toolchain)"""
        file_aux = open(self.path, "rb")
        text = file_aux.read(3)
        file_aux.close()
        if text == b'Xlx':
            return True
        else:
            return False


class VerilogFile(SourceFile):

    """This is the class providing the generic Verilog file"""

    def __init__(self, path, module, library=None,
                 include_dirs=None, is_include=False):
        SourceFile.__init__(self, path=path, module=module, library=library)
        from hdlmake.vlog_parser import VerilogParser
        self.include_dirs = []
        if include_dirs:
            self.include_dirs.extend(include_dirs)
        self.include_dirs.append(path_mod.relpath(self.dirname))
        self.parser = VerilogParser(self)
        for dir_aux in self.include_paths:
            self.parser.add_search_path(dir_aux)
        self.is_include = is_include


class SVFile(VerilogFile):
    """This is the class providing the generic SystemVerilog file"""
    pass


# TCL COMMAND FILE

class TCLFile(File):
    """This is the class providing the Tool Command Language file"""
    pass


# XILINX FILES

class UCFFile(File):
    """This is the class providing the User Constraint Guide file"""
    pass


class XISEFile(File):
    """This is the class providing the new Xilinx ISE project file"""
    pass


class CDCFile(File):
    """This is the class providing the Xilinx ChipScope Definition
    and Connection file"""
    pass


class XMPFile(File):
    """Xilinx Embedded Micro Processor"""
    pass


class PPRFile(File):
    """Xilinx PlanAhead Project"""
    pass


class XPRFile(File):
    """Xilinx Vivado Project"""
    pass


class BDFile(File):
    """Xilinx Block Design"""
    pass


class XCOFile(File):
    """Xilinx Core Generator File"""
    pass


class NGCFile(File):
    """Xilinx Generated Netlist File"""
    pass


class XDCFile(File):
    """Xilinx Design Constraint File"""
    pass


class XCFFile(File):
    """Xilinx XST Constraint File"""
    pass


class COEFile(File):
    """Xilinx Coefficient File"""
    pass


class MIFFile(File):
    """Xilinx Memory Initialization File"""
    pass


class RAMFile(File):
    """Xilinx RAM  File"""
    pass


class VHOFile(File):
    """Xilinx VHDL Template File"""
    pass


class BMMFile(File):
    """Xilinx Block Memory Map File"""
    pass


class VEOFile(File):
    """Xilinx Verilog Template File"""
    pass


class XCIFile(File):
    """Xilinx Core IP File"""
    pass


XILINX_FILE_DICT = {
    'xise': XISEFile,
    'ise': XISEFile,
    'ngc': NGCFile,
    'ucf': UCFFile,
    'cdc': CDCFile,
    'xmp': XMPFile,
    'ppr': PPRFile,
    'xpr': XPRFile,
    'bd': BDFile,
    'xco': XCOFile,
    'xdc': XDCFile,
    'xcf': XCFFile,
    'coe': COEFile,
    'mif': MIFFile,
    'ram': RAMFile,
    'vho': VHOFile,
    'veo': VEOFile,
    'bmm': BMMFile,
    'xci': XCIFile}


# SYNOPSYS FILES

class SDCFile(File):
    """Synopsys Design Constraints"""
    pass


# LATTICE FILES

class LDFFile(File):
    """Lattice Diamond Project File"""
    pass


class LPFFile(File):
    """Lattice Preference/Constraint File"""
    pass

class PCFFile(File):
    """Icestorm Physical constraints File"""
    pass

class EDFFile(File):
    """EDIF Netlist Files"""
    pass


LATTICE_FILE_DICT = {
    'ldf': LDFFile,
    'lpf': LPFFile,
    'edf': EDFFile,
    'edif': EDFFile,
    'edi': EDFFile,
    'edn': EDFFile,
    'pcf': PCFFile}


# MICROSEMI/ACTEL FILES

class PDCFile(File):
    """Physical Design Constraints"""
    pass


MICROSEMI_FILE_DICT = {
    'pdc': PDCFile}


# OHR FILES

class WBGenFile(File):
    """Wishbone generator file"""
    pass


# INTEL/ALTERA FILES

class QIPFile(File):
    """This is the class providing the Altera Quartus IP file"""
    pass


class QSYSFile(File):
    """Qsys - Altera's System Integration Tool"""
    pass


class DPFFile(File):
    """This is the class providing Altera Quartus Design Protocol File"""
    pass


class QSFFile(File):
    """Quartus Settings File"""
    pass


class BSFFile(File):
    """Quartus Block Symbol File"""
    pass


class BDFFile(File):
    """Quartus Block Design File"""
    pass


class TDFFile(File):
    """Quartus Text Design File"""
    pass


class GDFFile(File):
    """Quartus Graphic Design File"""
    pass


class SignalTapFile(File):
    """This is the class providing the Altera Signal Tap Language file"""
    pass


ALTERA_FILE_DICT = {
    'stp': SignalTapFile,
    'qip': QIPFile,
    'qsys': QSYSFile,
    'dpf': DPFFile,
    'qsf': QSFFile,
    'bsf': BSFFile,
    'bdf': BDFFile,
    'tdf': TDFFile,
    'gdf': GDFFile}


class SourceFileSet(set):

    """Class providing a extension of the 'set' object that includes
    methods that allow for an easier management of a collection of HDL
    source files"""

    def __init__(self):
        super(SourceFileSet, self).__init__()
        self = []

    def __str__(self):
        return str([str(f) for f in self])

    def add(self, files):
        """Add a set of files to the source fileset instance"""
        if isinstance(files, str):
            raise RuntimeError("Expected object, not a string")
        elif files is None:
            logging.debug("Got None as a file.\n Ommiting")
        else:
            try:
                for file_aux in files:
                    super(SourceFileSet, self).add(file_aux)
            except TypeError:  # single file, not a list
                super(SourceFileSet, self).add(files)

    def filter(self, filetype):
        """Method that filters and returns all of the HDL source files
        contained in the instance SourceFileSet matching the provided type"""
        out = SourceFileSet()
        for file_aux in self:
            if isinstance(file_aux, filetype):
                out.add(file_aux)
        return out

    def inversed_filter(self, filetype):
        """Method that filters and returns all of the HDL source files
        contained in the instance SourceFileSet NOT matching the provided
        type"""
        out = SourceFileSet()
        for file_aux in self:
            if not isinstance(file_aux, filetype):
                out.add(file_aux)
        return out

    def get_libs(self):
        """Method that returns a set containing all of the libraries that are
        provided by any of the source files in the SourceFileSet"""
        ret = set()
        for file_aux in self:
            try:
                ret.add(file_aux.library)
            except TypeError:
                pass
        return ret


def create_source_file(path, module, library=None,
                       include_dirs=None, is_include=False):
    """Function that analyzes the given arguments and returns a new HDL source
    file of the appropriated type"""
    if path is None or path == "":
        raise RuntimeError("Expected a file path, got: " + str(path))
    if not os.path.isabs(path):
        path = os.path.abspath(path)
    tmp = path.rsplit('.')
    extension = tmp[len(tmp) - 1]
    logging.debug("add file " + path)

    new_file = None
    if extension in ['vhd', 'vhdl', 'vho']:
        new_file = VHDLFile(path=path,
                            module=module,
                            library=library)
    elif extension in ['v', 'vh', 'vo', 'vm']:
        new_file = VerilogFile(path=path,
                               module=module,
                               library=library,
                               include_dirs=include_dirs,
                               is_include=is_include)
    elif extension == 'sv' or extension == 'svh':
        new_file = SVFile(path=path,
                          module=module,
                          library=library,
                          include_dirs=include_dirs,
                          is_include=is_include)
    elif extension == 'wb':
        new_file = WBGenFile(path=path, module=module)
    elif extension == 'tcl':
        new_file = TCLFile(path=path, module=module)
    elif extension == 'sdc':
        new_file = SDCFile(path=path, module=module)
    elif extension in XILINX_FILE_DICT:
        new_file = XILINX_FILE_DICT[extension](path=path, module=module)
    elif extension in ALTERA_FILE_DICT:
        new_file = ALTERA_FILE_DICT[extension](path=path, module=module)
    elif extension in LATTICE_FILE_DICT:
        new_file = LATTICE_FILE_DICT[extension](path=path, module=module)
    elif extension in MICROSEMI_FILE_DICT:
        new_file = MICROSEMI_FILE_DICT[extension](path=path, module=module)
    else:
        logging.error("Cannot create source file %s, "
                      "unknown file extension %s", path, extension)
        quit(1)
    return new_file



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2013 - 2015 CERN
# Author: Pawel Szostek (pawel.szostek@cern.ch)
# Multi-tool support by Javier D. Garcia-Lasheras (javier@garcialasheras.com)
#
# This file is part of Hdlmake.
#
# Hdlmake is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Hdlmake is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Hdlmake.  If not, see <http://www.gnu.org/licenses/>.
#

"""Module providing the classes that are used to handle Xilinx ISE"""

from __future__ import print_function
from __future__ import absolute_import
import logging

from .make_syn import ToolSyn

from hdlmake.util import shell
from hdlmake.srcfile import (VHDLFile, VerilogFile, SVFile,
                             UCFFile, CDCFile, NGCFile, BMMFile, XCOFile)

FAMILY_NAMES = {
    "XC6S": "Spartan6",
    "XC3S": "Spartan3",
    "XC6V": "Virtex6",
    "XC5V": "Virtex5",
    "XC4V": "Virtex4",
    "XC7Z": "Zynq",
    "XC7V": "Virtex7",
    "XC7K": "Kintex7",
    "XC7A": "Artix7"}

ISE_STANDARD_LIBS = ['ieee', 'ieee_proposed', 'iSE', 'simprims', 'std',
                     'synopsys', 'unimacro', 'unisim', 'XilinxCoreLib']


class ToolISE(ToolSyn):

    """Class providing the methods to create and build a Xilinx ISE project"""

    TOOL_INFO = {
        'name': 'ISE',
        'id': 'ise',
        'windows_bin': 'xtclsh.exe',
        'linux_bin': 'xtclsh',
        'project_ext': 'xise'}

    STANDARD_LIBS = ['ieee', 'ieee_proposed', 'iSE', 'simprims', 'std',
                     'synopsys', 'unimacro', 'unisim', 'XilinxCoreLib']

    SUPPORTED_FILES = {
        UCFFile: 'xfile add $(sourcefile)',
        CDCFile: 'xfile add $(sourcefile)',
        BMMFile: 'xfile add $(sourcefile)',
        NGCFile: 'xfile add $(sourcefile)',
        XCOFile: 'xfile add $(sourcefile)'}

    HDL_FILES = {
        VHDLFile: 'xfile add $(sourcefile)',
        VerilogFile: 'xfile add $(sourcefile)',
        SVFile: 'xfile add $(sourcefile)'}

    CLEAN_TARGETS = {'clean': ["xst", "xlnx_auto_0_xdb", "iseconfig _xmsgs",
                               "_ngo", "*.b", "*_summary.html",
                               "*.bld", "*.cmd_log", "*.drc", "*.lso", "*.ncd",
                               "*.ngc", "*.ngd", "*.ngr", "*.pad", "*.par",
                               "*.pcf", "*.prj", "*.ptwx", "*.stx", "*.syr",
                               "*.twr", "*.twx", "*.gise", "*.gise", "*.bgn",
                               "*.unroutes", "*.ut", "*.xpi", "*.xst",
                               "*.xise", "*.xwbt",
                               "*_envsettings.html", "*_guide.ncd",
                               "*_map.map", "*_map.mrp", "*_map.ncd",
                               "*_map.ngm", "*_map.xrpt", "*_ngdbuild.xrpt",
                               "*_pad.csv", "*_pad.txt", "*_par.xrpt",
                               "*_summary.xml", "*_usage.xml", "*_xst.xrpt",
                               "usage_statistics_webtalk.html", "webtalk.log",
                               "par_usage_statistics.html", "webtalk_pn.xml"],
                     'mrproper': ["*.bit", "*.bin", "*.mcs"]}

    _ISE_RUN = '''\
$(TCL_OPEN)
set process {{{0}}}
process run '$$'process
set result [process get '$$'process status]
if {{ '$$'result == \\"errors\\" }} {{
    exit 1
}}
$(TCL_SAVE)
$(TCL_CLOSE)'''

    TCL_CONTROLS = {
        'create': 'project new $(PROJECT_FILE)',
        'open': 'project open $(PROJECT_FILE)',
        'save': 'project save',
        'close': 'project close',
        'project': '$(TCL_CREATE)\n'
                   'xfile remove [search \* -type file]\n'
                   'source files.tcl\n'
                   '{0}\n'
                   'project set top $(TOP_MODULE)\n'
                   '$(TCL_SAVE)\n'
                   '$(TCL_CLOSE)',
        'synthesize': _ISE_RUN.format("Synthesize - XST"),
        'translate': _ISE_RUN.format("Translate"),
        'map': _ISE_RUN.format("Map"),
        'par': _ISE_RUN.format("Place "
            + ("^&" if shell.check_windows() else "'&'")
            + " Route"),
        'bitstream': _ISE_RUN.format("Generate Programming File"),
        'install_source': "*.bit *.bin"}

    def __init__(self):
        super(ToolISE, self).__init__()
        self._tool_info.update(ToolISE.TOOL_INFO)
        self._hdl_files.update(ToolISE.HDL_FILES)
        self._supported_files.update(ToolISE.SUPPORTED_FILES)
        self._standard_libs.extend(ToolISE.STANDARD_LIBS)
        self._clean_targets.update(ToolISE.CLEAN_TARGETS)
        self._tcl_controls.update(ToolISE.TCL_CONTROLS)

    def _makefile_syn_top(self):
        """Create the top part of the synthesis Makefile for ISE"""
        syn_family = self.manifest_dict.get("syn_family", None)
        if syn_family is None:
            syn_family = FAMILY_NAMES.get(
                self.manifest_dict["syn_device"][0:4].upper())
            if syn_family is None:
                logging.error(
                    "syn_family is not defined in Manifest.py"
                    " and can not be guessed!")
                quit(-1)
        self.manifest_dict["syn_family"] = syn_family
        super(ToolISE, self)._makefile_syn_top()


    def _makefile_syn_tcl(self):
        """Create a Xilinx synthesis project by TCL"""
        syn_properties = self.manifest_dict.get("syn_properties")
        project_new = []
        project_tcl = self._tcl_controls["project"]
        if shell.check_windows():
            tmp = 'project set "{0}" "{1}"'
        else:
            tmp = 'project set \\"{0}\\" \\"{1}\\"'
        properties = [
            ['family', '$(SYN_FAMILY)'],
            ['device', '$(SYN_DEVICE)'],
            ['package', '$(SYN_PACKAGE)'],
            ['speed', '$(SYN_GRADE)'],
            ['Manual Implementation Compile Order', 'false'],
            ['Auto Implementation Top', 'false'],
            ['Create Binary Configuration File', 'true']]
        if not syn_properties is None:
            properties.extend(syn_properties)
        for prop in properties:
            project_new.append(tmp.format(prop[0], prop[1]))
        project_new.append('set compile_directory .')
        self._tcl_controls["project"] = project_tcl.format(
            "\n".join(project_new))
        super(ToolISE, self)._makefile_syn_tcl()



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2013 - 2015 CERN
# Author: Pawel Szostek (pawel.szostek@cern.ch)
# Multi-tool support by Javier D. Garcia-Lasheras (javier@garcialasheras.com)
#
# This file is part of Hdlmake.
#
# Hdlmake is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Hdlmake is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Hdlmake.  If not, see <http://www.gnu.org/licenses/>.
#

"""Module providing support for Lattice Diamond IDE"""


from __future__ import absolute_import
from .make_syn import ToolSyn
from hdlmake.srcfile import EDFFile, LPFFile, VHDLFile, VerilogFile


class ToolDiamond(ToolSyn):

    """Class providing the interface for Lattice Diamond synthesis"""

    TOOL_INFO = {
        'name': 'Diamond',
        'id': 'diamond',
        'windows_bin': 'pnmainc.exe',
        'linux_bin': 'diamondc',
        'project_ext': 'ldf'}

    STANDARD_LIBS = ['ieee', 'std']

    _LATTICE_SOURCE = 'prj_src {0} $(sourcefile)'

    SUPPORTED_FILES = {
        EDFFile: _LATTICE_SOURCE.format('add'),
        LPFFile: _LATTICE_SOURCE.format('add -exclude') + '; ' +
                 _LATTICE_SOURCE.format('enable')}

    HDL_FILES = {
        VHDLFile: _LATTICE_SOURCE.format('add'),
        VerilogFile: _LATTICE_SOURCE.format('add')}

    CLEAN_TARGETS = {'clean': ["*.sty", "$(PROJECT)"],
                     'mrproper': ["*.jed"]}

    TCL_CONTROLS = {'create': 'prj_project new -name $(PROJECT)'
                              ' -impl $(PROJECT)'
                              ' -dev {0} -synthesis \"synplify\"',
                    'open': 'prj_project open $(PROJECT).ldf',
                    'save': 'prj_project save',
                    'close': 'prj_project close',
                    'project': '$(TCL_CREATE)\n'
                               'source files.tcl\n'
                               '$(TCL_SAVE)\n'
                               '$(TCL_CLOSE)',
                    'par': '$(TCL_OPEN)\n'
                           'prj_run PAR -impl $(PROJECT)\n'
                           '$(TCL_SAVE)\n'
                           '$(TCL_CLOSE)',
                    'bitstream': '$(TCL_OPEN)\n'
                                 'prj_run Export'
                                 ' -impl $(PROJECT) -task Bitgen\n'
                                 '$(TCL_SAVE)\n'
                                 '$(TCL_CLOSE)',
                    'install_source': '$(PROJECT)/$(PROJECT)_$(PROJECT).jed'}

    def __init__(self):
        super(ToolDiamond, self).__init__()
        self._tool_info.update(ToolDiamond.TOOL_INFO)
        self._hdl_files.update(ToolDiamond.HDL_FILES)
        self._supported_files.update(ToolDiamond.SUPPORTED_FILES)
        self._standard_libs.extend(ToolDiamond.STANDARD_LIBS)
        self._clean_targets.update(ToolDiamond.CLEAN_TARGETS)
        self._tcl_controls.update(ToolDiamond.TCL_CONTROLS)

    def _makefile_syn_tcl(self):
        """Create a Diamond synthesis project by TCL"""
        syn_device = self.manifest_dict["syn_device"]
        syn_grade = self.manifest_dict["syn_grade"]
        syn_package = self.manifest_dict["syn_package"]
        create_tmp = self._tcl_controls["create"]
        target = syn_device + syn_grade + syn_package
        self._tcl_controls["create"] = create_tmp.format(target.upper())
        super(ToolDiamond, self)._makefile_syn_tcl()



action = "synthesis"

syn_device = "xc7z020"
syn_grade = "-1"
syn_package = "clg484"
syn_top = "zedboard_top"
syn_project = "zedboard_top"
syn_tool = "vivado"

modules = {
  "local" : [ "../../../top/zedboard/verilog" ],
}





#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2013 - 2015 CERN
# Author: Pawel Szostek (pawel.szostek@cern.ch)
# Multi-tool support by Javier D. Garcia-Lasheras (javier@garcialasheras.com)
#
# This file is part of Hdlmake.
#
# Hdlmake is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Hdlmake is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Hdlmake.  If not, see <http://www.gnu.org/licenses/>.
#

"""Module providing support for GHDL simulator"""

from __future__ import absolute_import
import string

from .make_sim import ToolSim
from hdlmake.srcfile import VHDLFile


class ToolGHDL(ToolSim):

    """Class providing the interface for GHDL simulator"""

    TOOL_INFO = {
        'name': 'GHDL',
        'id': 'ghdl',
        'windows_bin': None,
        'linux_bin': 'ghdl'}

    STANDARD_LIBS = ['ieee', 'std']

    HDL_FILES = {VHDLFile: ''}

    CLEAN_TARGETS = {'clean': ["*.cf", "*.o", "$(TOP_MODULE)", "work"],
                     'mrproper': ["*.vcd"]}

    SIMULATOR_CONTROLS = {'vlog': None,
                          'vhdl': '$(GHDL) -a $(GHDL_OPT) $<',
                          'compiler': '$(GHDL) -e $(GHDL_OPT) $(TOP_MODULE)'}

    def __init__(self):
        super(ToolGHDL, self).__init__()
        self._tool_info.update(ToolGHDL.TOOL_INFO)
        self._hdl_files.update(ToolGHDL.HDL_FILES)
        self._standard_libs.extend(ToolGHDL.STANDARD_LIBS)
        self._clean_targets.update(ToolGHDL.CLEAN_TARGETS)
        self._simulator_controls.update(ToolGHDL.SIMULATOR_CONTROLS)

    def _makefile_sim_options(self):
        """Print the GHDL options to the Makefile"""
        ghdl_opt = self.manifest_dict.get("ghdl_opt", '')
        ghdl_string = string.Template(
            "GHDL := ghdl\n"
            "GHDL_OPT := ${ghdl_opt}\n")
        self.writeln(ghdl_string.substitute(
            ghdl_opt=ghdl_opt))

    def _makefile_sim_compilation(self):
        """Print the GDHL simulation compilation target"""
        self.writeln("simulation: $(VERILOG_OBJ) $(VHDL_OBJ)")
        self.writeln("\t\t" + self._simulator_controls['compiler'])
        self.writeln('\n')
        self._makefile_sim_dep_files()



action = "simulation"
sim_tool = "active_hdl"
sim_top = "counter_tb"

sim_post_cmd = "vsimsa -do ../play_sim.do; avhdl wave.asdb"

modules = {
  "local" : [ "../../../testbench/counter_tb/verilog" ],
}




#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2013-2016 CERN
# Author: Pawel Szostek (pawel.szostek@cern.ch)
# Multi-tool support by Javier D. Garcia-Lasheras (javier@garcialasheras.com)
#
# This file is part of Hdlmake.
#
# Hdlmake is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Hdlmake is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Hdlmake.  If not, see <http://www.gnu.org/licenses/>.

"""
This python module is the one in charge to support the HDLMake modules
It includes the base Module class, that inherits several action
specific parent modules providing specific methods and attributes.

"""

from __future__ import print_function
from __future__ import absolute_import
import os
import logging

from hdlmake.util import path as path_mod
from hdlmake.util import shell
from hdlmake.manifest_parser import ManifestParser
from .content import ModuleContent, ModuleArgs
import six


class Module(ModuleContent):

    """
    This is the class providing the HDLMake module, the basic element
    providing the modular behavior allowing for structured designs.
    """

    def __init__(self, module_args, pool):
        """Calculate and initialize the origin attributes: path, source..."""
        assert module_args.url is not None
        assert module_args.source is not None
        super(Module, self).__init__()
        self.init_config(module_args)
        self.set_pool(pool)
        self.module_args = ModuleArgs()
        self.module_args = module_args

    def __str__(self):
        return self.module_args.url

    @property
    def is_fetched_to(self):
        """Get the path where the module instance resides"""
        return os.path.dirname(self.path)

    def submodules(self):
        """Get a list with all the submodules this module instance requires"""
        def __nonull(submodule_list):
            """Returns a list with the submodules, being empty if null"""
            if not submodule_list:
                return []
            else:
                return submodule_list
        return __nonull(self.local) + __nonull(self.git) \
            + __nonull(self.gitsm) + __nonull(self.svn)

    def remove_dir_from_disk(self):
        """Delete the module dir if it is already fetched and available"""
        if not self.isfetched:
            return
        logging.debug("Removing " + self.path)
        command_tmp = shell.rmdir_command() + " " + self.path
        shell.run(command_tmp)

    def process_manifest(self):
        """
        This method processes the different manifest dictionary sections
        contained in the action specific inherited Python modules.
        """
        logging.debug("Process manifest at: " + os.path.dirname(self.path))
        super(Module, self).process_manifest()

    def get_include_dirs_list(self):
        """Private method that processes the included directory list"""
        # Include dirs
        include_dirs = []
        if "include_dirs" in self.manifest_dict:
            if isinstance(self.manifest_dict["include_dirs"],
                          six.string_types):
                dir_list = path_mod.compose(
                    self.path, self.manifest_dict["include_dirs"])
                include_dirs.append(dir_list)
            else:
                dir_list = [path_mod.compose(x, self.path) for
                            x in self.manifest_dict["include_dirs"]]
                include_dirs.extend(dir_list)
            # Analyze included dirs and report if any issue is found
            for dir_ in include_dirs:
                if path_mod.is_abs_path(dir_):
                    logging.warning("%s contains absolute path to an include "
                                    "directory: %s", self.path, dir_)
                if not os.path.exists(dir_):
                    logging.warning(self.path +
                                    " has an unexisting include directory: " +
                                    dir_)
        return include_dirs

    def parse_manifest(self):
        """
        Create a dictionary from the module Manifest.py and assign it
        to the manifest_dict property.
        In order to do this, it creates a ManifestParser object and
        feeds it with:
        - the arbitrary code from pool's top_module options
            (it assumes a top_module exists before any parsing!)
        - the Manifest.py (if exists)
        - the extra_context:
          - If this is the root module (has not parent),
              use an empty extra_context in the parser
          - If this is a submodule (has a parent),
              inherit the extra_context as:
            - the full manifest_dict from the top_module...
            - ...but deleting some key fields that needs to be respected.
        """

        if self.manifest_dict or self.isfetched is False:
            return
        if self.path is None:
            raise RuntimeError()

        logging.debug("""
***********************************************************
PARSE START: %s
***********************************************************""", self.path)

        manifest_parser = ManifestParser()

        manifest_parser.add_prefix_code(
            self.pool.options.prefix_code)
        manifest_parser.add_sufix_code(
            self.pool.options.sufix_code)

        parser_tmp = manifest_parser.add_manifest(self.path)

        if not parser_tmp == None:

            if self.parent is None:
                extra_context = {}
            else:
                extra_context = dict(self.top_module.manifest_dict)
            extra_context["__manifest"] = self.path

            # The parse method is where most of the parser action takes place!
            opt_map = None
            try:
                opt_map = manifest_parser.parse(extra_context=extra_context)
            except NameError as name_error:
                logging.error(
                    "Error while parsing {0}:\n{1}: {2}.".format(
                        self.path, type(name_error), name_error))
                quit(1)
            self.manifest_dict = opt_map
        else:
            self.manifest_dict = {}

        # Process the parsed manifest_dict to assign the module properties
        self.process_manifest()
        self.process_git_submodules()

        # Parse every detected submodule
        for module_aux in self.submodules():
            module_aux.parse_manifest()

        logging.debug("""
***********************************************************
PARSE END: %s
***********************************************************

                      """, self.path)



action = "synthesis"

syn_device = "1k"
syn_grade = "dummy"
syn_package = "tq144"
syn_top = "icestick_top"
syn_project = "demo"
syn_tool = "icestorm"

modules = {
  "local" : [ "../../../top/icestick/verilog" ]
}




#!/usr/bin/python
#
# Copyright (c) 2013 CERN
# Author: Pawel Szostek (pawel.szostek@cern.ch)
#
# This file is part of Hdlmake.
#
# Hdlmake is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Hdlmake is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Hdlmake.  If not, see <http://www.gnu.org/licenses/>.
#

"""Module providing the Classes used to provide and handle dependable files"""

from __future__ import absolute_import
from __future__ import print_function
import os
import logging

from .util import path as path_mod
import six


class DepRelation(object):

    """Class used to create instances representing HDL dependency relations"""

    # direction
    PROVIDE = 1
    USE = 2

    # rel_type
    ENTITY = 1
    PACKAGE = 2
    INCLUDE = 3
    ARCHITECTURE = 4
    MODULE = ENTITY

    def __init__(self, obj_name, direction, rel_type):
        assert direction in [DepRelation.PROVIDE, DepRelation.USE]
        assert rel_type in [
            DepRelation.ENTITY,
            DepRelation.PACKAGE,
            DepRelation.INCLUDE,
            DepRelation.ARCHITECTURE,
            DepRelation.MODULE]
        self.direction = direction
        self.rel_type = rel_type
        self.obj_name = obj_name.lower()

    def satisfies(self, rel_b):
        """Check if the current dependency relation matches the provided one"""
        if (rel_b.direction == DepRelation.PROVIDE or
                self.direction == DepRelation.USE):
            return False
        if rel_b.rel_type == self.rel_type and rel_b.obj_name == self.obj_name:
            return True
        return False

    def library(self):
        """If the current relation type is PACKAGE, it returns the base name of
        the library, e.g. for work.counter it returns work."""
        if self.rel_type == DepRelation.PACKAGE:
            libdotpackage = self.obj_name
            try:
                return libdotpackage.split('.')[0]
            except ValueError:
                return None
        else:
            return None

    def __repr__(self):
        dstr = {self.USE: "Use", self.PROVIDE: "Provide"}
        ostr = {
            self.ENTITY: "entity",
            self.PACKAGE: "package",
            self.INCLUDE: "include/header",
            self.ARCHITECTURE: "architecture",
            self.MODULE: "module"}
        return "%s %s '%s'" % (dstr[self.direction],
                               ostr[self.rel_type],
                               self.obj_name)

    def __hash__(self):
        return hash(self.__repr__())

    def __eq__(self, other):
        return (isinstance(other, self.__class__)
                and self.__dict__ == other.__dict__)

    def __ne__(self, other):
        return not self.__eq__(other)


class File(object):

    """This is the base class for all of the different files in HDLMake"""

    def __init__(self, path, module=None):
        self.path = path
        assert not isinstance(module, six.string_types)
        self.module = module

    @property
    def name(self):
        """Property defined as a method that gets the basename of the file
        path, i.e. it strips the path and takes the full file name"""
        return os.path.basename(self.path)

    @property
    def purename(self):
        """Property defined as a method that gets the name of the file
        and strips put the extension from the file"""
        return os.path.splitext(self.name)[0]

    @property
    def dirname(self):
        """Property defined as a method that gets the name of the directory
        in which the file is stored"""
        return os.path.dirname(self.path)

    def rel_path(self, directory=None):
        """Returns the relative path for the file calculated with (directory)
        as the origin reference -- if none, it will be defaulted to current
        folder from which we are launching the program"""
        if directory is None:
            directory = os.getcwd()
        return path_mod.relpath(self.path, directory)

    def __str__(self):
        return self.path

    def __eq__(self, other):
        _not_found = object()
        path_self, path_other = [getattr(obj, "path", _not_found)
                                 for obj in [self, other]]
        if path_self is _not_found or path_other is _not_found:
            return False
        elif path_self != path_other:
            return False
        return True

    def __hash__(self):
        return hash(self.path)

    def __cmp__(self, other):
        if self.path < other.path:
            return -1
        if self.path == other.path:
            return 0
        if self.path > other.path:
            return 1

    def __ne__(self, other):
        return not self.__eq__(other)

    def isdir(self):
        """Check if the defined file path is a directory"""
        return os.path.isdir(self.path)

    def show(self):
        """Print the file path to stdout"""
        print(self.path)

    def extension(self):
        """Method that gets the extension for the file instance"""
        tmp = self.path.rsplit('.')
        ext = tmp[len(tmp) - 1]
        return ext


class DepFile(File):

    """Class that serves as base to all those HDL files that can be
    parsed and solved (Verilog, SystemVerilog, VHDL)"""

    def __init__(self, file_path, module):
        assert isinstance(file_path, six.string_types)
        File.__init__(self, path=file_path, module=module)
        self.file_path = file_path
        self.rels = set()
        self.depends_on = set()
        self.dep_level = None
        self.is_parsed = False
        self.file_path = file_path
        self.include_paths = []

    def add_relation(self, rel):
        """Add a new relation to the set provided by the file"""
        self.rels.add(rel)

    def satisfies(self, rel_b):
        """Check if any of the file object relations match any of the relations
        listed in the parameter (rel_b)"""
        assert isinstance(rel_b, DepRelation)
        # self._parse_if_needed()
        return any([x.satisfies(rel_b) for x in self.rels])

    def show_relations(self):
        """Print the file relations to stdout: can be used for logging"""
        # self._parse_if_needed()
        for relation in self.rels:
            print(str(relation))

    @property
    def filename(self):
        """Property defined as a method that checks the basename of the file
        path in the host, i.e. the name of the last directory on the path"""
        return os.path.basename(self.file_path)

    def get_dep_level(self):
        """Get the dependency level for the file instance, so we can order
        later the full fileset"""
        if self.dep_level is None:
            if len(self.depends_on) == 0:
                self.dep_level = 0
            else:
                # set dep_level to a negative value so we can detect
                # if the recusion below brings us back to
                # this file in a circular reference, that would otherwise
                # result in an infinite loop.
                self.dep_level = -1
                # recurse, to find the largest number of levels below.
                self.dep_level = 1 + \
                    max([dep.get_dep_level() for dep in self.depends_on])
        elif self.dep_level < 0:
            logging.warning("Probably run into a circular reference of file "
                            "dependencies. It appears %s depends on itself, "
                            "indirectly via atleast one other file.",
                            self.file_path)
        return self.dep_level



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2013 - 2015 CERN
# Author: Pawel Szostek (pawel.szostek@cern.ch)
# Modified to allow ISim simulation by Lucas Russo (lucas.russo@lnls.br)
# Modified to allow ISim simulation by Adrian Byszuk (adrian.byszuk@lnls.br)
# Multi-tool support by Javier D. Garcia-Lasheras (javier@garcialasheras.com)
#
# This file is part of Hdlmake.
#
# Hdlmake is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Hdlmake is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Hdlmake.  If not, see <http://www.gnu.org/licenses/>.
#

"""Module providing support for Xilinx ISim simulator"""

from __future__ import absolute_import
import os
import os.path
import logging

from .make_sim import ToolSim
from hdlmake.util import shell
from hdlmake.srcfile import VerilogFile, VHDLFile


class ToolISim(ToolSim):

    """Class providing the interface for Xilinx ISim simulator"""

    TOOL_INFO = {
        'name': 'ISim',
        'id': 'isim',
        'windows_bin': 'isimgui.exe',
        'linux_bin': 'isimgui'}

    STANDARD_LIBS = ['std', 'ieee', 'ieee_proposed', 'vl', 'synopsys',
                     'simprim', 'unisim', 'unimacro', 'aim', 'cpld',
                     'pls', 'xilinxcorelib', 'aim_ver', 'cpld_ver',
                     'simprims_ver', 'unisims_ver', 'uni9000_ver',
                     'unimacro_ver', 'xilinxcorelib_ver', 'secureip']

    HDL_FILES = {VerilogFile: '', VHDLFile: ''}

    CLEAN_TARGETS = {'clean': ["xilinxsim.ini $(LIBS)", "fuse.xmsgs",
                               "fuse.log", "fuseRelaunch.cmd", "isim",
                               "isim.log", "isim.wdb", "isim_proj",
                               "isim_proj.*"],
                     'mrproper': ["*.vcd"]}

    def __init__(self):
        super(ToolISim, self).__init__()
        self._tool_info.update(ToolISim.TOOL_INFO)
        self._hdl_files.update(ToolISim.HDL_FILES)
        self._standard_libs.extend(ToolISim.STANDARD_LIBS)
        self._clean_targets.update(ToolISim.CLEAN_TARGETS)

    def _makefile_sim_top(self):
        """Print the top section of the Makefile for Xilinx ISim"""

        def __get_xilinxsim_ini_dir():
            """Get Xilinx ISim ini simulation file"""
            if "sim_path" in self.manifest_dict:
                xilinx_dir = str(os.path.join(
                    self.manifest_dict["sim_path"], "..", ".."))
            else:
                logging.error("Cannot calculate xilinx tools base directory")
                quit(1)
            hdl_language = 'vhdl'  # 'verilog'
            if shell.check_windows():
                os_prefix = 'nt'
            else:
                os_prefix = 'lin'
            if shell.architecture() == 32:
                arch_sufix = ''
            else:
                arch_sufix = '64'
            xilinx_ini_path = str(os.path.join(xilinx_dir,
                                  hdl_language,
                                  "hdp",
                                  os_prefix + arch_sufix))
            # Ensure the path is absolute and normalized
            return os.path.abspath(xilinx_ini_path)
        self.writeln("""## variables #############################
PWD := $(shell pwd)
TOP_MODULE := """ + self.manifest_dict.get("sim_top", '') + """
FUSE_OUTPUT ?= isim_proj

XILINX_INI_PATH := """ + __get_xilinxsim_ini_dir() +
                     """
""")

    def _makefile_sim_options(self):
        """Print the Xilinx ISim simulation options in the Makefile"""
        def __get_rid_of_isim_incdirs(vlog_opt):
            """Clean the vlog options from include dirs"""
            if not vlog_opt:
                vlog_opt = ""
            vlogs = vlog_opt.split(' ')
            ret = []
            skip = False
            for vlog_option in vlogs:
                if skip:
                    skip = False
                    continue
                if not vlog_option.startswith("-i"):
                    ret.append(vlog_option)
                else:
                    skip = True
            return ' '.join(ret)
        default_options = ("-intstyle default -incremental " +
                           "-initfile xilinxsim.ini ")
        self.writeln("VHPCOMP_FLAGS := " +
            default_options + self.manifest_dict.get("vcom_opt", ''))
        self.writeln("VLOGCOMP_FLAGS := " +
            default_options + __get_rid_of_isim_incdirs(
            self.manifest_dict.get("vlog_opt", '')))

    def _makefile_sim_compilation(self):
        """Print the compile simulation target for Xilinx ISim"""
        fileset = self.fileset
        libs = set(f.library for f in fileset)
        self.write('LIBS := ')
        self.write(' '.join(libs))
        self.write('\n')
        # tell how to make libraries
        self.write('LIB_IND := ')
        self.write(' '.join([lib + shell.slash_char() +
            "." + lib for lib in libs]))
        self.write('\n')
        self.writeln("""\
simulation: xilinxsim.ini $(LIB_IND) $(VERILOG_OBJ) $(VHDL_OBJ) fuse
$(VERILOG_OBJ): $(LIB_IND) xilinxsim.ini
$(VHDL_OBJ): $(LIB_IND) xilinxsim.ini

""")
        self.writeln("xilinxsim.ini: $(XILINX_INI_PATH)" +
            shell.slash_char() + "xilinxsim.ini")
        self.writeln("\t\t" + shell.copy_command() + " $< .")
        self.writeln("""\
fuse:
\t\tfuse work.$(TOP_MODULE) -intstyle ise -incremental -o $(FUSE_OUTPUT)

""")
        # ISim does not have a vmap command to insert additional libraries in
        #.ini file.
        for lib in libs:
            self.write(lib + shell.slash_char() + "." + lib + ":\n")
            self.write(
                ' '.join(["\t(" + shell.mkdir_command(), lib, "&&",
                          shell.touch_command(),
                          lib + shell.slash_char() + "." + lib + " "]))
            # self.write(' '.join(["&&", "echo", "\""+lib+"="+lib+"/."+lib+"\"
            # ", ">>", "xilinxsim.ini) "]))
            self.write(
                ' '.join(["&&",
                          "echo",
                          lib + "=" + lib,
                          " >>",
                          "xilinxsim.ini) "]))
            self.write(' '.join(["||", shell.del_command(), lib, "\n"]))
            self.write('\n')
            # Modify xilinxsim.ini file by including the extra local libraries
            # self.write(' '.join(["\t(echo """, lib+"="+lib+"/."+lib, ">>",
            # "${XILINX_INI_PATH}/xilinxsim.ini"]))
        # rules for all _primary.dat files for sv
        # incdir = ""
        objs = []
        for vl_file in fileset.filter(VerilogFile):
            comp_obj = os.path.join(vl_file.library, vl_file.purename)
            objs.append(comp_obj)
            # self.write(os.path.join(vl_file.library, vl_file.purename,
            #            '.'+vl_file.purename+"_"+vl_file.extension())+': ')
            # self.writeln(".PHONY: " + os.path.join(comp_obj,
            # '.'+vl_file.purename+"_"+vl_file.extension()))
            self.write(
                os.path.join(
                    comp_obj,
                    '.' +
                    vl_file.purename +
                    "_" +
                    vl_file.extension(
                    )) +
                ': ')
            self.write(vl_file.rel_path() + ' ')
            self.writeln(
                ' '.join([fname.rel_path() for fname in vl_file.depends_on]))
            self.write("\t\tvlogcomp -work " + vl_file.library
                       + "=." + shell.slash_char() + vl_file.library)
            self.write(" $(VLOGCOMP_FLAGS) ")
            # if isinstance(vl_file, SVFile):
            #    self.write(" -sv ")
            # incdir = "-i "
            # incdir += " -i ".join(vl_file.include_dirs)
            # incdir += " "
            if vl_file.include_dirs:
                self.write(' -i ')
                self.write(' '.join(vl_file.include_dirs))
            self.writeln(" $<")
            self.write("\t\t@" + shell.mkdir_command() + " $(dir $@)")
            self.writeln(" && " + shell.touch_command() + " $@ \n\n")
        self.write("\n")
        # list rules for all _primary.dat files for vhdl
        for vhdl_file in fileset.filter(VHDLFile):
            lib = vhdl_file.library
            purename = vhdl_file.purename
            comp_obj = os.path.join(lib, purename)
            objs.append(comp_obj)
            # each .dat depends on corresponding .vhd file and its dependencies
            # self.write(os.path.join(lib, purename, "."+purename+"_"
            #     + vhdl_file.extension()) + ": "+ vhdl_file.rel_path()+" "
            #     + os.path.join(lib, purename, "."+purename) + '\n')
            # self.writeln(".PHONY: " + os.path.join(comp_obj,
            # "."+purename+"_"+ vhdl_file.extension()))
            self.write(
                os.path.join(
                    comp_obj,
                    "." + purename + "_" + vhdl_file.extension(
                    )) + ": " + vhdl_file.rel_path(
                ) + " " + os.path.join(
                    lib,
                    purename,
                    "." + purename) + '\n')
            self.writeln(
                ' '.join(["\t\tvhpcomp $(VHPCOMP_FLAGS)",
                          "-work",
                          lib + "=." + shell.slash_char() + lib,
                          "$< "]))
            self.writeln("\t\t@" + shell.mkdir_command() +
                         " $(dir $@) && " + shell.touch_command() + " $@\n")
            self.writeln()
            # dependency meta-target.
            # This rule just list the dependencies of the above file
            # if len(vhdl_file.depends_on) != 0:
            # self.writeln(".PHONY: " + os.path.join(
            #     lib, purename, "."+purename))
            # Touch the dependency file as well. In this way, "make" will
            # recompile only what is needed (out of date)
            # if len(vhdl_file.depends_on) != 0:
            self.write(os.path.join(lib, purename, "." + purename) + ":")
            for dep_file in vhdl_file.depends_on:
                if dep_file in fileset:
                    name = dep_file.purename
                    self.write(
                        " \\\n" + os.path.join(dep_file.library,
                                               name, "." + name + "_" +
                                               dep_file.extension()))
                else:
                    self.write(" \\\n" + os.path.join(dep_file.rel_path()))
            self.write('\n')
            self.writeln("\t\t@" + shell.mkdir_command() +
                         " $(dir $@) && " + shell.touch_command() + " $@\n")



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2013 - 2015 CERN
# Author: Pawel Szostek (pawel.szostek@cern.ch)
# Multi-tool support by Javier D. Garcia-Lasheras (javier@garcialasheras.com)
#
# This file is part of Hdlmake.
#
# Hdlmake is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Hdlmake is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Hdlmake.  If not, see <http://www.gnu.org/licenses/>.
#

"""Module providing support for Aldec Active-HDL simulator"""

from __future__ import absolute_import
from .make_sim import ToolSim
from hdlmake.srcfile import VHDLFile, VerilogFile, SVFile


class ToolActiveHDL(ToolSim):

    """Class providing the interface to control an Active-HDL simulation"""

    TOOL_INFO = {
        'name': 'Aldec Active-HDL',
        'id': 'active_hdl',
        'windows_bin': 'vsimsa.exe',
        'linux_bin': None}

    STANDARD_LIBS = ['ieee', 'std']

    HDL_FILES = {VHDLFile: None, VerilogFile: None, SVFile: None}

    CLEAN_TARGETS = {'clean': ["run.command", "library.cfg", "work"],
                     'mrproper': ["*.vcd", "*.asdb"]}

    def __init__(self):
        super(ToolActiveHDL, self).__init__()
        self._tool_info.update(ToolActiveHDL.TOOL_INFO)
        self._hdl_files.update(ToolActiveHDL.HDL_FILES)
        self._standard_libs.extend(ToolActiveHDL.STANDARD_LIBS)
        self._clean_targets.update(ToolActiveHDL.CLEAN_TARGETS)

    def _makefile_sim_compilation(self):
        """Print Makefile compilation target for Aldec Active-HDL simulator"""
        fileset = self.fileset
        self.writeln("simulation:")
        self.writeln("\t\techo # Active-HDL command file,"
                     " generated by HDLMake > run.command")
        self.writeln()
        self.writeln("\t\techo # Create library and set as"
                     " default target >> run.command")
        self.writeln("\t\techo alib work >> run.command")
        self.writeln("\t\techo set worklib work >> run.command")
        self.writeln()
        self.writeln(
            "\t\techo # Compiling HDL source files >> run.command")
        for vl_file in fileset.filter(VerilogFile):
            self.writeln(
                "\t\techo alog \"" +
                vl_file.rel_path(
                ) +
                "\" >> run.command")
        for sv_file in fileset.filter(SVFile):
            self.writeln(
                "\t\techo alog \"" +
                sv_file.rel_path(
                ) +
                "\" >> run.command")
        for vhdl_file in fileset.filter(VHDLFile):
            self.writeln(
                "\t\techo acom \"" +
                vhdl_file.rel_path(
                ) +
                "\" >> run.command")
        self.writeln()
        self.writeln("\t\tvsimsa -do run.command")



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2013 CERN
# Author: Pawel Szostek (pawel.szostek@cern.ch)
#
# This file is part of Hdlmake.
#
# Hdlmake is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Hdlmake is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Hdlmake.  If not, see <http://www.gnu.org/licenses/>.
#

"""This module provides a set of functions that are commonly used in HDLMake"""

from __future__ import print_function
from __future__ import absolute_import
import os


def url_parse(url):
    """
    Check if link to a Git repo seems to be correct. Filter revision
    number and branch
    """
    url_clean, branch, rev = None, None, None
    if "@@" in url:
        url_clean, rev = url.split("@@")
    elif "::" in url:
        url_clean, branch = url.split("::")
    else:
        url_clean = url

    return (url_clean, branch, rev)


def svn_parse(url):
    """
    Check if link to a SVN repo seems to be correct. Filter revision
    number
    """
    url_clean, rev = None, None
    if "@" in url:
        url_clean, rev = url.split("@")
    else:
        url_clean = url

    return (url_clean, rev)


def url_basename(url):
    """
    Get basename from an url
    """
    if url.endswith(".git"):
        parts = url[:-4].split("/")
    elif url[-1] == '/':
        parts = url[:-1].split("/")
    else:
        parts = url.split("/")
    ret = parts[-1]
    return ret


def svn_basename(url):
    """
    Get basename from an SVN url
    """
    words = url.split('//')
    try:
        words = words[1].split('/')
        return '/'.join(words[1:])
    except IndexError:
        return None


def pathsplit(path, rest=None):
    """
    Split the provided path and return as a tuple
    """
    if rest is None:
        rest = []
    (head, tail) = os.path.split(path)
    if len(head) < 1:
        return [tail] + rest
    if len(tail) < 1:
        return [head] + rest
    return pathsplit(head, [tail] + rest)


def commonpath(path1, path2, common=None):
    """
    Return the common path for the provided paths
    """
    if common is None:
        common = []
    if len(path1) < 1:
        return (common, path1, path2)
    if len(path2) < 1:
        return (common, path1, path2)
    if path1[0] != path2[0]:
        return (common, path1, path2)
    return commonpath(path1[1:], path2[1:], common + [path1[0]])


def is_rel_path(path):
    """Check if the given path is relative"""
    return not os.path.isabs(path)


def is_abs_path(path):
    """Check if the given path is absolute"""
    return os.path.isabs(path)


def relpath(path1, path2=None):
    """Return the relative path of one path with respect to the other"""
    if path2 is None:
        path2 = os.getcwd()
    if path1 == path2:
        return '.'
    return os.path.relpath(path1, path2)


def rel2abs(path, base=None):
    """
    converts a relative path to an absolute path.

    @param path the path to convert - if already absolute, is returned
    without conversion.
    @param base - optional. Defaults to the current directory.
    The base is intelligently concatenated to the given relative path.
    @return the relative path of path from base
    """
    if base is None:
        base = os.getcwd()
    if os.path.isabs(path):
        return path
    retval = os.path.join(base, path)
    return os.path.abspath(retval)


def compose(path, base=None):
    """Get the relative path composition of the provided path"""
    if base is None:
        base = os.getcwd()
    return os.path.relpath(os.path.abspath(
        os.path.join(base, path)))


def flatten_list(sth):
    """Convert the argument in a list, being an empty list if none"""
    if sth is not None:
        if not isinstance(sth, (list, tuple)):
            sth = [sth]
    else:
        sth = []
    return sth



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2013 - 2015 CERN
# Author: Pawel Szostek (pawel.szostek@cern.ch)
# Multi-tool support by Javier D. Garcia-Lasheras (javier@garcialasheras.com)
#
# This file is part of Hdlmake.
#
# Hdlmake is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Hdlmake is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Hdlmake.  If not, see <http://www.gnu.org/licenses/>.

"""This is the entry point for HDLMake working in command line app mode"""

from __future__ import print_function
from __future__ import absolute_import
import argparse
import sys

from .manifest_parser import ManifestParser
from .module_pool import ModulePool
from ._version import __version__


def main():
    """This is the main function, where HDLMake starts.
    Here, we make the next processes:
        -- parse command
        -- check and set the environment
        -- prepare the global module containing the heavy common stuff
    """

    #
    # SET & GET PARSER
    #
    parser = _get_parser()

    #
    # PARSE & GET OPTIONS
    #
    options = _get_options(sys, parser)

    # Create a ModulePool object, this will become our workspace
    modules_pool = ModulePool(options)

    # Execute the appropriated action for the freshly created modules pool
    _action_runner(modules_pool)


def _action_runner(modules_pool):
    """Funtion that decodes and executed the action selected by the user"""
    options = modules_pool.options
    if options.command == "manifest-help":
        ManifestParser().print_help()
        quit(0)
    elif options.command == "makefile":
        modules_pool.makefile()
    elif options.command == "fetch":
        modules_pool.fetch()
    elif options.command == "clean":
        modules_pool.clean()
    elif options.command == "list-mods":
        modules_pool.list_modules()
    elif options.command == "list-files":
        modules_pool.list_files()
    elif options.command == "tree":
        modules_pool.generate_tree()


def _get_parser():
    """This is the parser function, where options and commands are defined.
    Here, we make the next processes:
    """
    description = ("A tool designed to help FPGA designers to manage "
                 + "and share their HDL code by automatically finding file "
                 + "dependencies, writing synthesis & simulation Makefiles, "
                 + "and fetching IP-Core libraries from remote repositories.")
    parser = argparse.ArgumentParser("hdlmake", description=description)
    subparsers = parser.add_subparsers(title="commands", dest="command")
    makefile = subparsers.add_parser(
        "makefile",
        help="write the Makefile (default action for hdlmake)")
    makefile.add_argument(
        "-f", "--filename",
        help="name for the Makefile file to be created",
        default=None,
        dest="filename")
    subparsers.add_parser(
        "fetch",
        help="fetch and/or update all of the remote modules")
    subparsers.add_parser(
        "clean",
        help="clean all of the already fetched remote modules")
    listmod = subparsers.add_parser(
        "list-mods",
        help="list all modules together with their files")
    listmod.add_argument(
        "--with-files",
        help="list modules together with their files",
        default=False,
        action="store_true",
        dest="withfiles")
    listmod.add_argument(
        "--terse",
        help="do not print comments",
        default=False,
        action="store_true",
        dest="terse")
    listfiles = subparsers.add_parser(
        "list-files",
        help="list all of the files in the design hierarchy")
    listfiles.add_argument(
        "--delimiter",
        help="set delimitier for the list of files",
        dest="delimiter",
        default=None)
    listfiles.add_argument(
        "--reverse",
        help="reverse the order for the list of files",
        dest="reverse",
        default=False,
        action="store_true")
    listfiles.add_argument(
        "--top",
        help="print only those files required to build 'top'",
        dest="top",
        default=None)
    tree = subparsers.add_parser(
        "tree",
        help="generate a module hierarchy tree graph")
    tree.add_argument(
        "--with-files",
        help="add files to the module hierarchy tree",
        default=False,
        action="store_true",
        dest="withfiles")
    tree.add_argument(
        "--mode",
        dest="mode",
        default="mods",
        help="set the working mode for the tree generator: "
             "(mods, dfs, bfs)")
    subparsers.add_parser(
        "manifest-help",
        help="print manifest file variables description")
    parser.add_argument(
        '-v',
        '--version',
        action='version',
        help="print the version of this program",
        version=parser.prog +
        " " +
        __version__)
    parser.add_argument(
        '-a',
        '--all',
        action='store_true',
        dest="all_files",
        help="use all the listed files, do not solve the fileset")
    parser.add_argument(
        "--log",
        dest="log",
        default="info",
        help="logging level: debug, info, warning, error, critical")
    parser.add_argument(
        "--logfile",
        help="path to the optional log file",
        default=None,
        dest="logfile")
    parser.add_argument(
        "-p", "--prefix",
        dest="prefix_code",
        default="",
        help="Python code executed before every Manifest.py")
    parser.add_argument(
        "-s", "--sufix",
        dest="sufix_code",
        default="",
        help="Python code executed after every Manifest.py")
    return parser


def _get_options(sys_aux, parser):
    """Function that decodes and set the provided command user options"""
    options = None
    if len(sys_aux.argv[1:]) == 0:
        options = parser.parse_args(['makefile'])
    elif len(sys_aux.argv[1:]) == 2 and (
             sys_aux.argv[1] == '-f' or sys_aux.argv[1] == '--filename'):
        options = parser.parse_args(['makefile'] + sys_aux.argv[1:])
    else:
        options = parser.parse_args(sys_aux.argv[1:])
    return options


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2013, 2014 CERN
# Author: Pawel Szostek (pawel.szostek@cern.ch)
# Multi-tool support by Javier D. Garcia-Lasheras (javier@garcialasheras.com)
#
# This file is part of Hdlmake.
#
# Hdlmake is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Hdlmake is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Hdlmake.  If not, see <http://www.gnu.org/licenses/>.
#

"""Package providing the bridge with the Host O.S. environment"""

from __future__ import print_function
from __future__ import absolute_import
import os
import os.path
import logging
import six


class Env(dict):

    """The Env (Environment) is a dictionary containing the environmental
    variables related with HDLMake for a proper use in the Python code"""

    def __init__(self, options):
        dict.__init__(self)
        self.options = options

    def _report_and_set_hdlmake_var(self, name):
        """Create a new entry in the Env dictionary and initialize the value
        to the obtained from the O.S. environmental variable if defined"""
        def _get(name):
            """Ask the Host O.S. for the value of an HDLMAKE_(name)
            environmental variable"""
            assert not name.startswith("HDLMAKE_")
            assert isinstance(name, six.string_types)
            name = name.upper()
            return os.environ.get("HDLMAKE_%s" % name)
        name = name.upper()
        val = _get(name)
        if val:
            logging.debug('Environmental variable HDLMAKE_%s is set: "%s".',
                          name, val)
            self[name.lower()] = val
            return True
        else:
            logging.warning("Environmental variable HDLMAKE_%s is not set.",
                            name)
            self[name.lower()] = None
            return False



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2013 - 2015 CERN
# Author: Pawel Szostek (pawel.szostek@cern.ch)
# Multi-tool support by Javier D. Garcia-Lasheras (javier@garcialasheras.com)
#
# This file is part of Hdlmake.
#
# Hdlmake is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Hdlmake is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Hdlmake.  If not, see <http://www.gnu.org/licenses/>.
#

"""Module providing support for Xilinx PlanAhead synthesis"""

from __future__ import absolute_import
from .xilinx import ToolXilinx
from hdlmake.srcfile import (UCFFile, NGCFile, XMPFile, XCOFile, BMMFile)


class ToolPlanAhead(ToolXilinx):

    """Class providing the interface for Xilinx PlanAhead synthesis"""

    TOOL_INFO = {
        'name': 'PlanAhead',
        'id': 'planahead',
        'windows_bin': 'planAhead.exe -mode tcl -source',
        'linux_bin': 'planAhead -mode tcl -source',
        'project_ext': 'ppr'}

    STANDARD_LIBS = ['ieee', 'ieee_proposed', 'simprims', 'std',
                     'synopsys', 'unimacro', 'unisim', 'XilinxCoreLib']

    SUPPORTED_FILES = {
        UCFFile: ToolXilinx._XILINX_SOURCE,
        NGCFile: ToolXilinx._XILINX_SOURCE,
        XMPFile: ToolXilinx._XILINX_SOURCE,
        BMMFile: ToolXilinx._XILINX_SOURCE,
        XCOFile: ToolXilinx._XILINX_SOURCE}

    CLEAN_TARGETS = {'clean': ["planAhead_*", "planAhead.*",
                               ".Xil", "$(PROJECT).cache", "$(PROJECT).data",
                               " $(PROJECT).runs", "$(PROJECT).ppr"]}

    TCL_CONTROLS = {'bitstream': '$(TCL_OPEN)\n'
                                 'launch_runs impl_1 -to_step Bitgen\n'
                                 'wait_on_run impl_1\n'
                                 '$(TCL_CLOSE)'}

    def __init__(self):
        super(ToolPlanAhead, self).__init__()
        self._tool_info.update(ToolPlanAhead.TOOL_INFO)
        self._supported_files.update(ToolPlanAhead.SUPPORTED_FILES)
        self._standard_libs.extend(ToolPlanAhead.STANDARD_LIBS)
        self._clean_targets.update(ToolPlanAhead.CLEAN_TARGETS)
        self._tcl_controls.update(ToolPlanAhead.TCL_CONTROLS)



"""This provides the stuff related with the HDLMake module,
from files to required submodules"""

from __future__ import absolute_import
import logging
from hdlmake import fetch
from hdlmake.util import path as path_mod
from .core import ModuleCore
import six
import os


class ModuleArgs(object):

    """This class is just a container for the main Module args"""

    def __init__(self):
        self.parent = None
        self.url = None
        self.source = fetch.LOCAL
        self.fetchto = None

    def set_args(self, parent, url, source, fetchto):
        """Set the module arguments"""
        self.parent = parent
        self.url = url
        if source == None:
            self.source = fetch.LOCAL
        else:
            self.source = source
        self.fetchto = fetchto

    def get_args(self):
        """Get the module arguments"""
        return self.parent, self.url, self.source, self.fetchto


class ModuleContent(ModuleCore):

    """Class providing the HDLMake module content"""

    def __init__(self):
        # Manifest Files Properties
        self.files = None
        # Manifest Modules Properties
        self.local = []
        self.git = []
        self.gitsm = []
        self.svn = []
        self.incl_makefiles = []
        super(ModuleContent, self).__init__()

    def process_manifest(self):
        """Process the content section of the manifest_dic"""
        super(ModuleContent, self).process_manifest()
        self._process_manifest_files()
        self._process_manifest_modules()
        self._process_manifest_makefiles()

    def _process_manifest_files(self):
        """Process the files instantiated by the HDLMake module"""
        from hdlmake.srcfile import SourceFileSet
        # HDL files provided by the module
        if "files" not in self.manifest_dict:
            self.files = SourceFileSet()
            try:
                logging.debug("No files in the manifest at %s",
                              self.path)
            except AttributeError:
                pass
        else:
            self.manifest_dict["files"] = path_mod.flatten_list(
                self.manifest_dict["files"])
            logging.debug("Files in %s: %s to library %s" ,
                          self.path,
                          str(self.manifest_dict["files"]),
                          self.library)
            paths = self._make_list_of_paths(self.manifest_dict["files"])
            self.files = self._create_file_list_from_paths(paths=paths)

    def _get_fetchto(self):
        """Calculate the fetchto folder"""
        if ("fetchto" in self.manifest_dict and
                self.manifest_dict["fetchto"] is not None):
            fetchto = path_mod.rel2abs(self.manifest_dict["fetchto"],
                                       self.path)
        else:
            fetchto = self.fetchto()
        return fetchto

    def _process_manifest_modules(self):
        """Process the submodules required by the HDLMake module"""
        # Process required modules
        fetchto = self._get_fetchto()
        if "modules" in self.manifest_dict:

            if "local" in self.manifest_dict["modules"]:
                local_paths = path_mod.flatten_list(
                    self.manifest_dict["modules"]["local"])
                local_mods = []
                for path in local_paths:
                    if path_mod.is_abs_path(path):
                        logging.error("Found an absolute path (" + path +
                                      ") in a manifest(" + self.path + ")")
                        quit(1)
                    path = path_mod.rel2abs(path, self.path)
                    local_mods.append(self.pool.new_module(parent=self,
                                                           url=path,
                                                           source=fetch.LOCAL,
                                                           fetchto=fetchto))
                self.local = local_mods
            else:
                self.local = []

            if "svn" in self.manifest_dict["modules"]:
                self.manifest_dict["modules"]["svn"] = path_mod.flatten_list(
                    self.manifest_dict["modules"]["svn"])
                svn_mods = []
                for url in self.manifest_dict["modules"]["svn"]:
                    svn_mods.append(self.pool.new_module(parent=self,
                                                         url=url,
                                                         source=fetch.SVN,
                                                         fetchto=fetchto))
                self.svn = svn_mods
            else:
                self.svn = []

            if "git" in self.manifest_dict["modules"]:
                self.manifest_dict["modules"]["git"] = path_mod.flatten_list(
                    self.manifest_dict["modules"]["git"])
                git_mods = []
                for url in self.manifest_dict["modules"]["git"]:
                    git_mods.append(self.pool.new_module(parent=self,
                                                         url=url,
                                                         source=fetch.GIT,
                                                         fetchto=fetchto))
                self.git = git_mods
            else:
                self.git = []

            if "gitsm" in self.manifest_dict["modules"]:
                self.manifest_dict["modules"]["gitsm"] = path_mod.flatten_list(
                    self.manifest_dict["modules"]["gitsm"])
                gitsm_mods = []
                for url in self.manifest_dict["modules"]["gitsm"]:
                    gitsm_mods.append(self.pool.new_module(parent=self,
                                                           url=url,
                                                           source=fetch.GITSM,
                                                           fetchto=fetchto))
                self.gitsm = gitsm_mods
            else:
                self.gitsm = []

    def process_git_submodules(self):
        """Get the submodules if found in the Manifest path"""
        if not self.source == fetch.GITSM:
            return
        git_submodule_dict = fetch.Git.get_git_submodules(self)
        git_toplevel = fetch.Git.get_git_toplevel(self)
        for submodule_key in git_submodule_dict.keys():
            url = git_submodule_dict[submodule_key]["url"]
            path = git_submodule_dict[submodule_key]["path"]
            path = os.path.join(git_toplevel, path)
            fetchto = os.path.sep.join(path.split(os.path.sep)[:-1])
            self.git.append(self.pool.new_module(parent=self,
                                                 url=url,
                                                 fetchto=fetchto,
                                                 source=fetch.GIT))

    def _process_manifest_makefiles(self):
        """Get the extra makefiles defined in the HDLMake module"""
        # Included Makefiles
        included_makefiles_aux = []
        if "incl_makefiles" in self.manifest_dict:
            if isinstance(self.manifest_dict["incl_makefiles"],
                    six.string_types):
                included_makefiles_aux.append(
                    self.manifest_dict["incl_makefiles"])
            else:  # list
                included_makefiles_aux = self.manifest_dict["incl_makefiles"][:]
        makefiles_paths = self._make_list_of_paths(included_makefiles_aux)
        self.incl_makefiles.extend(makefiles_paths)

    def _create_file_list_from_paths(self, paths):
        """
        Build a Source File Set containing the files indicated by the
        provided list of paths
        """
        from hdlmake.srcfile import create_source_file, SourceFileSet
        srcs = SourceFileSet()
        # Check if this is the top module and grab the include_dirs
        if self.parent is None:
            include_dirs = self.manifest_dict.get('include_dirs', [])
        else:
            include_dirs = self.top_module.manifest_dict.get('include_dirs', [])
        for path_aux in paths:
            if os.path.isdir(path_aux):
                dir_ = os.listdir(path_aux)
                for f_dir in dir_:
                    f_dir = os.path.join(self.path, path_aux, f_dir)
                    if not os.path.isdir(f_dir):
                        srcs.add(create_source_file(path=f_dir,
                                                    module=self,
                                                    library=self.library,
                                                    include_dirs=include_dirs))
            else:
                srcs.add(create_source_file(path=path_aux,
                                            module=self,
                                            library=self.library,
                                            include_dirs=include_dirs))
        return srcs



action = "simulation"
target = "xilinx"
sim_tool = "isim"
sim_top = "counter_tb"

sim_post_cmd = "./isim_proj -gui -tclbatch ../isim_cmd"

modules = {
  "local" : [ "../../../testbench/counter_tb/vhdl" ],
}





target = "xilinx"
action = "synthesis"

syn_device = "xc6slx45t"
syn_grade = "-3"
syn_package = "fgg484"
syn_top = "spec_top"
syn_project = "demo"
syn_tool = "planahead"

modules = {
  "local" : [ "../../../top/spec_v4/verilog" ],
}

